@@ -27,8 +27,9 @@ G_BEGIN_DECLS
 
 #define CONF_PREFIX "/apps/rhythmbox"
 
-#define CONF_FIRST_TIME CONF_PREFIX "/first_time_flag"
-#define CONF_AUTO_RATE  CONF_PREFIX "/auto_rate"
+#define CONF_FIRST_TIME CONF_PREFIX   "/first_time_flag"
+#define CONF_AUTO_RATE  CONF_PREFIX   "/auto_rate"
+#define CONF_GRACE_PERIOD CONF_PREFIX "/grace_period"
 #define CONF_UI_DIR               CONF_PREFIX "/ui"
 #define CONF_UI_STATUSBAR_HIDDEN  CONF_PREFIX "/ui/statusbar_hidden"
 #define CONF_UI_SOURCELIST_HIDDEN CONF_PREFIX "/ui/sourcelist_hidden"
@@ -38,8 +39,8 @@ G_BEGIN_DECLS
 #define CONF_UI_TIME_DISPLAY CONF_PREFIX "/ui/time_display"
 #define CONF_UI_BROWSER_VIEWS CONF_PREFIX "/ui/browser_views"
 #define CONF_STATE_PLAY_ORDER	CONF_PREFIX "/state/play_order"
-#define CONF_STATE_REPEAT	CONF_PREFIX "/state/repeat"
 #define CONF_STATE_VOLUME	CONF_PREFIX "/state/volume"
+#define CONF_STATE_BURN_DEVICE	CONF_PREFIX "/state/burn_device"
 
 G_END_DECLS
 
