@@ -56,6 +56,7 @@ typedef struct
 	void	(*complete)		(void);
 	void	(*entry_prop_changed)	(RhythmDBEntry *entry, RhythmDBPropType prop,
 					 const GValue *old, const GValue *new);
+	void    (*entry_removed)        (RhythmDBEntry *entry);
 
 } RhythmDBQueryModelClass;
 
@@ -75,7 +76,6 @@ void			rhythmdb_query_model_remove_entry	(RhythmDBQueryModel *model, RhythmDBEnt
 void			rhythmdb_query_model_set_connected	(RhythmDBQueryModel *model, gboolean connected);
 
 void			rhythmdb_query_model_signal_complete	(RhythmDBQueryModel *model);
-void			rhythmdb_query_model_finish_complete	(RhythmDBQueryModel *model);
 
 GnomeVFSFileSize	rhythmdb_query_model_get_size	(RhythmDBQueryModel *model);
 
