@@ -137,7 +137,9 @@ help_cb (GtkWidget *widget,
 	gnome_help_display ("rhythmbox.xml", "prefs", &error);
 
 	if (error != NULL) {
-		rb_error_dialog (error->message);
+		rb_error_dialog (NULL,
+				 _("Couldn't display help"),
+				 "%s", error->message);
 
 		g_error_free (error);
 	}
@@ -275,7 +277,7 @@ rb_shell_preferences_new (GList *views)
 	RBShellPreferences *shell_preferences;
 
 	shell_preferences = g_object_new (RB_TYPE_SHELL_PREFERENCES,
-				          NULL);
+				          NULL, NULL);
 
 	g_return_val_if_fail (shell_preferences->priv != NULL, NULL);
 
@@ -348,8 +350,9 @@ rb_shell_preferences_column_check_changed_cb (GtkCheckButton *butt,
 		colname = "RHYTHMDB_PROP_PLAY_COUNT";
 	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->last_played_check))
 		colname = "RHYTHMDB_PROP_LAST_PLAYED";
-	else if (butt == GTK_CHECK_BUTTON (shell_preferences->priv->quality_check))
-		colname = "RHYTHMDB_PROP_QUALITY";
+	else if (shell_preferences->priv->quality_check
+		 && butt == GTK_CHECK_BUTTON (shell_preferences->priv->quality_check))
+		colname = "RHYTHMDB_PROP_BITRATE";
 	else
 		g_assert_not_reached ();
 
@@ -410,9 +413,10 @@ rb_shell_preferences_sync (RBShellPreferences *shell_preferences)
 		gtk_toggle_button_set_active
 			(GTK_TOGGLE_BUTTON (shell_preferences->priv->last_played_check),
 			 strstr (columns, "RHYTHMDB_PROP_LAST_PLAYED") != NULL);
-		gtk_toggle_button_set_active
-			(GTK_TOGGLE_BUTTON (shell_preferences->priv->quality_check),
-			 strstr (columns, "RHYTHMDB_PROP_QUALITY") != NULL);
+		if (shell_preferences->priv->quality_check)
+			gtk_toggle_button_set_active
+				(GTK_TOGGLE_BUTTON (shell_preferences->priv->quality_check),
+				 strstr (columns, "RHYTHMDB_PROP_BITRATE") != NULL);
 	}
 
 	g_free (columns);
