@@ -20,6 +20,7 @@
  */
 
 #include <gtk/gtkhbox.h>
+#include <gtk/gtkactiongroup.h>
 
 #include "rb-shell-player.h"
 #include "rb-source.h"
@@ -54,13 +55,16 @@ typedef struct
 GType			rb_statusbar_get_type	(void);
 
 RBStatusbar *		rb_statusbar_new	(RhythmDB *db,
-						 BonoboUIComponent *component,
+						 GtkActionGroup *actiongroup,
 						 RBShellPlayer *player);
 
 void			rb_statusbar_set_source	(RBStatusbar *bar,
 						 RBSource *player);
 
-void			rb_statusbar_set_progress(RBStatusbar *bar, double progress);
+void			rb_statusbar_set_progress(RBStatusbar *bar, 
+						  double progress,
+						  const char *text);
+
 void			rb_statusbar_sync_state (RBStatusbar *statusbar);
 
 G_END_DECLS
