@@ -25,8 +25,6 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <glade/glade.h>
-#include <bonobo/bonobo-ui-component.h>
-#include <bonobo/bonobo-window.h>
 #include <string.h>
 
 #include "rb-source.h"
@@ -43,7 +41,6 @@
 #include "rb-file-helpers.h"
 #include "rb-dialog.h"
 #include "rb-volume.h"
-#include "rb-bonobo-helpers.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
 #include "rb-song-info.h"
@@ -63,6 +60,7 @@ static void rb_library_source_class_init (RBLibrarySourceClass *klass);
 static void rb_library_source_init (RBLibrarySource *source);
 static GObject *rb_library_source_constructor (GType type, guint n_construct_properties,
 					       GObjectConstructParam *construct_properties);
+static void rb_library_source_dispose (GObject *object);
 static void rb_library_source_finalize (GObject *object);
 static void rb_library_source_set_property (GObject *object,
 			                  guint prop_id,
@@ -72,15 +70,12 @@ static void rb_library_source_get_property (GObject *object,
 			                  guint prop_id,
 			                  GValue *value,
 			                  GParamSpec *pspec);
-static void rb_library_source_cmd_choose_genre (BonoboUIComponent *component,
-						RBShell *shell,
-						const char *verbname);
-static void rb_library_source_cmd_choose_artist (BonoboUIComponent *component,
-						 RBShell *shell,
-						 const char *verbname);
-static void rb_library_source_cmd_choose_album (BonoboUIComponent *component,
-						RBShell *shell,
-						const char *verbname);
+static void rb_library_source_cmd_choose_genre (GtkAction *action,
+						RBShell *shell);
+static void rb_library_source_cmd_choose_artist (GtkAction *action,
+						 RBShell *shell);
+static void rb_library_source_cmd_choose_album (GtkAction *action,
+						RBShell *shell);
 static void rb_library_source_handle_genre_selection (RBLibrarySource *libsource, GList *genres);
 static void rb_library_source_handle_artist_selection (RBLibrarySource *libsource, GList *artists);
 static void rb_library_source_handle_album_selection (RBLibrarySource *libsource, GList *albums);
@@ -115,7 +110,7 @@ static void rb_library_source_state_prefs_sync (RBLibrarySource *source);
 static void rb_library_source_ui_prefs_sync (RBLibrarySource *source);
 static void rb_library_source_preferences_sync (RBLibrarySource *source);
 /* source methods */
-static const char *impl_get_status (RBSource *source);
+static char *impl_get_status (RBSource *source);
 static const char *impl_get_browser_key (RBSource *source);
 static const char *impl_get_search_key (RBSource *source);
 static GdkPixbuf *impl_get_pixbuf (RBSource *source);
@@ -135,11 +130,14 @@ static void rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryT
 void rb_library_source_browser_views_activated_cb (GtkWidget *widget,
 						 RBLibrarySource *source);
 static GPtrArray * construct_query_from_selection (RBLibrarySource *source);
+static void songs_view_drag_data_received_cb (GtkWidget *widget,
+					      GdkDragContext *dc,
+					      gint x, gint y,
+					      GtkSelectionData *data,
+					      guint info, guint time,
+					      RBLibrarySource *source);
 
 
-#define LIBRARY_SOURCE_SONGS_POPUP_PATH "/popups/LibrarySongsList"
-#define LIBRARY_SOURCE_POPUP_PATH "/popups/LibrarySourceList"
-
 #define CONF_UI_LIBRARY_DIR CONF_PREFIX "/ui/library"
 #define CONF_UI_LIBRARY_BROWSER_VIEWS CONF_PREFIX "/ui/library/browser_views"
 #define CONF_STATE_LIBRARY_DIR CONF_PREFIX "/state/library"
@@ -150,6 +148,8 @@ static GPtrArray * construct_query_from_selection (RBLibrarySource *source);
 
 struct RBLibrarySourcePrivate
 {
+	gboolean disposed;
+	
 	RhythmDB *db;
 	
 	GtkWidget *browser;
@@ -166,7 +166,6 @@ struct RBLibrarySourcePrivate
 
 	gboolean lock;
 
-	char *status;
 	char *artist;
 	char *album;
 
@@ -186,24 +185,35 @@ struct RBLibrarySourcePrivate
 	
 	gboolean loading_prefs;
 
+	GtkActionGroup *action_group;
 	GtkWidget *config_widget;
 	GSList *browser_views_group;
-
+	
 	RhythmDBEntryType entry_type;
+	
+	guint ui_dir_notify_id;
+	guint state_dir_notify_id;
+	guint browser_view_notify_id;
 };
 
-static BonoboUIVerb rb_library_source_verbs[] =
+static GtkActionEntry rb_library_source_actions [] =
 {
-	BONOBO_UI_VERB ("SLChooseGenre",(BonoboUIVerbFn) rb_library_source_cmd_choose_genre),
-	BONOBO_UI_VERB ("SLChooseArtist",(BonoboUIVerbFn) rb_library_source_cmd_choose_artist),
-	BONOBO_UI_VERB ("SLChooseAlbum", (BonoboUIVerbFn) rb_library_source_cmd_choose_album),
-	BONOBO_UI_VERB_END
+	{ "LibrarySrcChooseGenre", NULL, N_("Browse This _Genre"), NULL,
+	  N_("Set the browser to view only this genre"),
+	  G_CALLBACK (rb_library_source_cmd_choose_genre) },
+	{ "LibrarySrcChooseArtist", NULL , N_("Browse This _Artist"), NULL,
+	  N_("Set the browser to view only this artist"),
+	  G_CALLBACK (rb_library_source_cmd_choose_artist) },
+	{ "LibrarySrcChooseAlbum", NULL, N_("Browse This A_lbum"), NULL,
+	  N_("Set the browser to view only this album"),
+	  G_CALLBACK (rb_library_source_cmd_choose_album) }
 };
 
+static const GtkTargetEntry songs_view_drag_types[] = {{  "text/uri-list", 0, 0 }};
+
 enum
 {
 	PROP_0,
-	PROP_DB,	
 	PROP_ICON,
 	PROP_ENTRY_TYPE
 };
@@ -247,6 +257,7 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	object_class->dispose = rb_library_source_dispose;
 	object_class->finalize = rb_library_source_finalize;
 	object_class->constructor = rb_library_source_constructor;
 
@@ -276,13 +287,6 @@ rb_library_source_class_init (RBLibrarySourceClass *klass)
 	source_class->impl_receive_drag = impl_receive_drag;
 	source_class->impl_show_popup = impl_show_popup;
 
-	g_object_class_install_property (object_class,
-					 PROP_DB,
-					 g_param_spec_object ("db",
-							      "RhythmDB",
-							      "RhythmDB database",
-							      RHYTHMDB_TYPE,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 	g_object_class_install_property (object_class,
 					 PROP_ICON,
 					 g_param_spec_object ("icon",
@@ -309,58 +313,61 @@ update_browser_views_visibility (RBLibrarySource *source)
 	GtkWidget *genres = GTK_WIDGET (source->priv->genres);
 	GtkWidget *artists = GTK_WIDGET (source->priv->artists);
 	GtkWidget *albums = GTK_WIDGET (source->priv->albums);
-	BonoboUIComponent *component;
+	GtkAction *action;
 
 	views = eel_gconf_get_integer (CONF_UI_LIBRARY_BROWSER_VIEWS);
 
-	g_object_get (G_OBJECT (source), "component", &component, NULL);
+	gtk_widget_show (genres);
+	action = gtk_action_group_get_action (source->priv->action_group,
+					      "LibrarySrcChooseGenre");
+	if (action) {
+		g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
+	}
+
+
+	gtk_widget_show (artists);	
+	action = gtk_action_group_get_action (source->priv->action_group,
+					      "LibrarySrcChooseArtist"); 
+	if (action) {
+		g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
+	}
+
+	gtk_widget_show (albums);
+	action = gtk_action_group_get_action (source->priv->action_group,
+					      "LibrarySrcChooseAlbum"); 
+	if (action) {
+		g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
+	}
 	
 	switch (views)
 	{
-		case 0:
-			gtk_widget_hide (genres);
-			rb_library_source_handle_genre_selection (source, NULL);
-			rb_bonobo_set_sensitive (component,
-						 "/commands" "/SLChooseGenre",
-						 FALSE);
-			gtk_widget_show (artists);
-			rb_bonobo_set_sensitive (component,
-						 "/commands" "/SLChooseArtist",
-						 TRUE);
-			gtk_widget_show (albums);
-			rb_bonobo_set_sensitive (component,
-						 "/commands" "/SLChooseAlbum",
-						 TRUE);
-			/* Since this is hidden now, reset the query */
+	case 0:
+		action = gtk_action_group_get_action (source->priv->action_group,
+						      "LibrarySrcChooseGenre");
+		if (action) {
+			g_object_set (G_OBJECT (action), 
+				      "sensitive", FALSE, 
+				      NULL);
+		}
+		gtk_widget_hide (genres);
+		rb_library_source_handle_genre_selection (source, NULL);
+		/* Since this is hidden now, reset the query */
 		break;
-		case 1:
-			gtk_widget_show (genres);
-			rb_bonobo_set_sensitive (component,
-						 "/commands" "/SLChooseGenre",
-						 TRUE);
-			gtk_widget_show (artists);
-			rb_bonobo_set_sensitive (component,
-						 "/commands" "/SLChooseArtist",
-						 TRUE);
-			gtk_widget_hide (albums);
-			rb_library_source_handle_artist_selection (source, NULL);
-			rb_bonobo_set_sensitive (component,
-						 "/commands" "/SLChooseAlbum",
-						 FALSE);
+	case 1:
+		action = gtk_action_group_get_action (source->priv->action_group,
+						      "LibrarySrcChooseAlbum");
+		if (action) {
+			g_object_set (G_OBJECT (action), 
+				      "sensitive", FALSE, 
+				      NULL);
+		}
+		gtk_widget_hide (albums);
+		rb_library_source_handle_album_selection (source, NULL);
 		break;
-		case 2:
-			gtk_widget_show (genres);
-			rb_bonobo_set_sensitive (component,
-						 "/commands" "/SLChooseGenre",
-						 TRUE);
-			gtk_widget_show (artists);
-			rb_bonobo_set_sensitive (component,
-						 "/commands" "/SLChooseArtist",
-						 TRUE);
-			gtk_widget_show (albums);
-			rb_bonobo_set_sensitive (component,
-						 "/commands" "/SLChooseAlbum",
-						 TRUE);
+	case 2:
+		/* All are shown */
+		break;
+	default:
 		break;
 	}
 }
@@ -403,6 +410,19 @@ rb_library_source_init (RBLibrarySource *source)
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
 }
 
+static void
+rb_library_source_dispose (GObject *object)
+{
+	RBLibrarySource *source;
+	source = RB_LIBRARY_SOURCE (object);
+
+	if (source->priv->disposed)
+		return;
+	source->priv->disposed = TRUE;
+
+	g_object_unref (source->priv->db);
+}
+
 static void
 rb_library_source_finalize (GObject *object)
 {
@@ -416,8 +436,11 @@ rb_library_source_finalize (GObject *object)
 	g_return_if_fail (source->priv != NULL);
 
 	rb_debug ("finalizing library source");
+	
+	eel_gconf_notification_remove (source->priv->ui_dir_notify_id);
+	eel_gconf_notification_remove (source->priv->state_dir_notify_id);
+	eel_gconf_notification_remove (source->priv->browser_view_notify_id);
 
-	g_free (source->priv->status);
 	g_free (source->priv->artist);
 	g_free (source->priv->album);
 
@@ -432,39 +455,48 @@ rb_library_source_finalize (GObject *object)
 
 static void
 rb_library_source_songs_show_popup_cb (RBEntryView *view,
-				       RBLibrarySource *library_source)
+				       RBLibrarySource *source)
 {
-	GtkWidget *menu;
-	GtkWidget *window;
-
-	window = gtk_widget_get_ancestor (GTK_WIDGET (view),
-					  BONOBO_TYPE_WINDOW);
-
-	menu = gtk_menu_new ();
-
-	bonobo_window_add_popup (BONOBO_WINDOW (window), GTK_MENU (menu),
-			         LIBRARY_SOURCE_SONGS_POPUP_PATH);
-
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-			3, gtk_get_current_event_time ());
-
-	gtk_object_sink (GTK_OBJECT (menu));
+	_rb_source_show_popup (RB_SOURCE (source), "/LibraryViewPopup");
 }
 
-/* This function may seem a bit weird. Since we have 2 instances of objects 
- * inheriting from RBLibrarySource, we can't install those ui verbs when
- * creating a new instance of such an object. Attaching those verbs in the 
- * shell would also be weird imo since this would add more code 
- * dependant on the source type to the shell */
-void
-rb_library_source_class_add_verbs (RBShell *shell, 
-				   BonoboUIComponent *component)
+static void
+register_action_group (RBLibrarySource *source)
 {
-	bonobo_ui_component_add_verb_list_with_data (component, 
-						     rb_library_source_verbs,
-						     shell);
-}
+	GtkUIManager *uimanager;
+	GList *actiongroups;
+	GList *group;
+	RBShell *shell;
 
+	g_object_get (G_OBJECT (source), "ui-manager", &uimanager, NULL);
+	actiongroups = gtk_ui_manager_get_action_groups (uimanager);
+
+	/* Don't create the action group if it's already registered */
+	for (group = actiongroups; group != NULL; group = group->next) {
+		const gchar *name;
+
+		name = gtk_action_group_get_name (GTK_ACTION_GROUP (group->data));
+		if (strcmp (name, "LibraryActions") == 0) {
+			g_object_unref (G_OBJECT (uimanager));
+			source->priv->action_group = GTK_ACTION_GROUP (group->data);
+			return;
+		}
+	}
+
+	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
+	source->priv->action_group = gtk_action_group_new ("LibraryActions");
+	gtk_action_group_set_translation_domain (source->priv->action_group,
+						 GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (source->priv->action_group, 
+				      rb_library_source_actions,
+				      G_N_ELEMENTS (rb_library_source_actions),
+				      shell);
+	gtk_ui_manager_insert_action_group (uimanager, 
+					    source->priv->action_group, 0);
+	g_object_unref (G_OBJECT (uimanager));
+	g_object_unref (G_OBJECT (shell));
+
+}
 
 static GObject *
 rb_library_source_constructor (GType type, guint n_construct_properties,
@@ -473,15 +505,20 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	RBLibrarySource *source;
 	RBLibrarySourceClass *klass;
 	GObjectClass *parent_class;  
-	BonoboUIComponent *component;
+	RBShell *shell;
 
 	klass = RB_LIBRARY_SOURCE_CLASS (g_type_class_peek (RB_TYPE_LIBRARY_SOURCE));
 
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
+
 	source = RB_LIBRARY_SOURCE (parent_class->constructor (type, n_construct_properties,
 							       construct_properties));
 
-	g_object_get (G_OBJECT (source), "component", &component, NULL);
+	register_action_group (source);
+
+	g_object_get (G_OBJECT (source), "shell", &shell, NULL);
+	g_object_get (G_OBJECT (shell), "db", &source->priv->db, NULL);
+	g_object_unref (G_OBJECT (shell));
 
 	source->priv->paned = gtk_vpaned_new ();
 
@@ -508,6 +545,20 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 				 G_CALLBACK (songs_view_sort_order_changed_cb),
 				 source, 0);
 
+	/* set up drag and drop for the song tree view.
+	 * we don't use RBEntryView's DnD support because it does too much.
+	 * we just want to be able to drop songs in to add them to the 
+	 * library.
+	 */
+	g_signal_connect_object (G_OBJECT (source->priv->songs), 
+				 "drag_data_received",
+				 G_CALLBACK (songs_view_drag_data_received_cb),
+				 source, 0);
+	gtk_drag_dest_set (GTK_WIDGET (source->priv->songs),
+			   GTK_DEST_DEFAULT_ALL,
+			   songs_view_drag_types, 1,
+			   GDK_ACTION_COPY | GDK_ACTION_MOVE);	/* really accept move actions? */
+
 	/* set up genres treeview */
 	source->priv->genres = rb_property_view_new (source->priv->db, RHYTHMDB_PROP_GENRE,
 						     _("Genre"));
@@ -566,12 +617,15 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	rb_library_source_state_prefs_sync (source);
 	rb_library_source_ui_prefs_sync (source);
 	update_browser_views_visibility (source);
-	eel_gconf_notification_add (CONF_STATE_LIBRARY_DIR,
+	source->priv->ui_dir_notify_id =
+		eel_gconf_notification_add (CONF_STATE_LIBRARY_DIR,
 				    (GConfClientNotifyFunc) rb_library_source_state_pref_changed,
 				    source);
-	eel_gconf_notification_add (CONF_UI_LIBRARY_DIR,
+	source->priv->state_dir_notify_id =
+		eel_gconf_notification_add (CONF_UI_LIBRARY_DIR,
 				    (GConfClientNotifyFunc) rb_library_source_ui_pref_changed, source);
-	eel_gconf_notification_add (CONF_UI_LIBRARY_BROWSER_VIEWS,
+	source->priv->browser_view_notify_id =
+		eel_gconf_notification_add (CONF_UI_LIBRARY_BROWSER_VIEWS,
 				    (GConfClientNotifyFunc) rb_library_source_browser_views_changed, source);
 	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_ALL);
 	return G_OBJECT (source);
@@ -588,9 +642,6 @@ rb_library_source_set_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_DB:
-		source->priv->db = g_value_get_object (value);
-		break;
 	case PROP_ICON:
 		if (source->priv->pixbuf) {
 			g_object_unref (G_OBJECT (source->priv->pixbuf));
@@ -617,9 +668,6 @@ rb_library_source_get_property (GObject *object,
 
 	switch (prop_id)
 	{
-	case PROP_DB:
-		g_value_set_object (value, source->priv->db);
-		break;
 	case PROP_ICON:
 		g_value_set_object (value, source->priv->pixbuf);
 		break;
@@ -633,8 +681,7 @@ rb_library_source_get_property (GObject *object,
 }
 
 RBSource *
-rb_library_source_new (RBShell *shell, RhythmDB *db, 
-		       BonoboUIComponent *component)
+rb_library_source_new (RBShell *shell)
 {
 	RBSource *source;
 	GtkWidget *dummy = gtk_tree_view_new ();
@@ -649,8 +696,7 @@ rb_library_source_new (RBShell *shell, RhythmDB *db,
 					  "name", _("Library"),
 					  "internal-name", "<library>",
 					  "entry-type", RHYTHMDB_ENTRY_TYPE_SONG,
-					  "db", db,
-					  "component", component,
+					  "shell", shell,
 					  "icon", icon,
 					  NULL));
 	rb_shell_register_entry_type_for_source (shell, source, 
@@ -704,6 +750,12 @@ rb_library_source_gather_hash_keys (char *key, gpointer unused,
 	*data = g_list_prepend (*data, key);
 }
 
+static void
+rb_library_source_free_list_data (gpointer *data, gpointer unused)
+{
+	g_free (data);
+}
+
 static GList *
 rb_library_source_gather_properties (RBLibrarySource *source,
 				     RhythmDBPropType prop)
@@ -711,16 +763,14 @@ rb_library_source_gather_properties (RBLibrarySource *source,
 	GList *selected, *tem;
 	GHashTable *selected_set;
 
-	rhythmdb_read_lock (source->priv->db);
 	selected_set = g_hash_table_new (g_str_hash, g_str_equal);
 	selected = rb_entry_view_get_selected_entries (source->priv->songs);
 	for (tem = selected; tem; tem = tem->next) {
-		char *val = g_strdup (rhythmdb_entry_get_string (source->priv->db,
-								 tem->data, prop));
+		RhythmDBEntry *entry = tem->data;
+		char *val = g_strdup (rhythmdb_entry_get_string (entry, prop));
 		g_hash_table_insert (selected_set, val, NULL);
 	}
 
-	rhythmdb_read_unlock (source->priv->db);
 
 	g_list_free (selected);
 	
@@ -732,11 +782,10 @@ rb_library_source_gather_properties (RBLibrarySource *source,
 }
 
 static void
-rb_library_source_cmd_choose_genre (BonoboUIComponent *component,
-				    RBShell *shell,
-				    const char *verbname)
+rb_library_source_cmd_choose_genre (GtkAction *action,
+				    RBShell *shell)
 {
-	GList *props;	
+	GList *props;
 	RBLibrarySource *source;
 
 	rb_debug ("choosing genre");
@@ -744,13 +793,14 @@ rb_library_source_cmd_choose_genre (BonoboUIComponent *component,
 	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
 	props = rb_library_source_gather_properties (source, RHYTHMDB_PROP_GENRE);
 	rb_property_view_set_selection (source->priv->genres, props);
+	g_list_foreach (props, (GFunc) rb_library_source_free_list_data, NULL);
 	g_list_free (props);
+	g_object_unref (source);
 }
 
 static void
-rb_library_source_cmd_choose_artist (BonoboUIComponent *component,
-				     RBShell *shell,
-				     const char *verbname)
+rb_library_source_cmd_choose_artist (GtkAction *action,
+				     RBShell *shell)
 {
 	GList *props;	
 	RBLibrarySource *source;
@@ -760,13 +810,14 @@ rb_library_source_cmd_choose_artist (BonoboUIComponent *component,
 	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
 	props = rb_library_source_gather_properties (source, RHYTHMDB_PROP_ARTIST);
 	rb_property_view_set_selection (source->priv->artists, props);
+	g_list_foreach (props, (GFunc) rb_library_source_free_list_data, NULL);
 	g_list_free (props);
+	g_object_unref (source);
 }
 
 static void
-rb_library_source_cmd_choose_album (BonoboUIComponent *component,
-				    RBShell *shell,
-				    const char *verbname)
+rb_library_source_cmd_choose_album (GtkAction *action,
+				    RBShell *shell)
 {
 	GList *props;	
 	RBLibrarySource *source;
@@ -776,7 +827,9 @@ rb_library_source_cmd_choose_album (BonoboUIComponent *component,
 	g_object_get (G_OBJECT (shell), "selected-source", &source, NULL);
 	props = rb_library_source_gather_properties (source, RHYTHMDB_PROP_ALBUM);
 	rb_property_view_set_selection (source->priv->albums, props);
+	g_list_foreach (props, (GFunc) rb_library_source_free_list_data, NULL);
 	g_list_free (props);
+	g_object_unref (source);
 }
 
 static void
@@ -889,15 +942,16 @@ songs_view_sort_order_changed_cb (RBEntryView *view, RBLibrarySource *source)
 	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_SEARCH);
 }
 
-static const char *
+static char *
 impl_get_status (RBSource *asource)
 {
 	RBLibrarySource *source = RB_LIBRARY_SOURCE (asource);
-	g_free (source->priv->status);
-	source->priv->status = rhythmdb_compute_status_normal (rb_entry_view_get_num_entries (source->priv->songs),
-							       rb_entry_view_get_duration (source->priv->songs),
-							       rb_entry_view_get_total_size (source->priv->songs));
-	return source->priv->status;
+	gchar *status;
+
+	status = rhythmdb_compute_status_normal (rb_entry_view_get_num_entries (source->priv->songs),
+						 rb_entry_view_get_duration (source->priv->songs),
+						 rb_entry_view_get_total_size (source->priv->songs));
+	return status;
 }
 
 static const char *
@@ -1047,12 +1101,7 @@ impl_get_artist (RBSource *asource)
 	entry = rb_entry_view_get_playing_entry (source->priv->songs);
 
 	if (entry != NULL) {
-		rhythmdb_read_lock (source->priv->db);
-		
-		source->priv->artist =
-			g_strdup (rhythmdb_entry_get_string (source->priv->db, entry, RHYTHMDB_PROP_ARTIST));
-
-		rhythmdb_read_unlock (source->priv->db);
+		source->priv->artist = g_strdup (rb_refstring_get (entry->artist));
 	} else {
 		source->priv->artist = NULL;
 	}
@@ -1071,12 +1120,7 @@ impl_get_album (RBSource *asource)
 	entry = rb_entry_view_get_playing_entry (source->priv->songs);
 
 	if (entry != NULL) {
-		rhythmdb_read_lock (source->priv->db);
-
-		source->priv->album =
-			g_strdup (rhythmdb_entry_get_string (source->priv->db, entry, RHYTHMDB_PROP_ALBUM));
-		
-		rhythmdb_read_unlock (source->priv->db);
+		source->priv->album = g_strdup (rb_refstring_get (entry->album));
 	} else {
 		source->priv->album = NULL;
 	}
@@ -1117,9 +1161,8 @@ impl_delete (RBSource *asource)
 	GList *l;
 
 	for (l = rb_entry_view_get_selected_entries (source->priv->songs); l != NULL; l = g_list_next (l)) {
-		rhythmdb_write_lock (source->priv->db);
 		rhythmdb_entry_delete (source->priv->db, l->data);
-		rhythmdb_write_unlock (source->priv->db);
+		rhythmdb_commit (source->priv->db);
 	}
 }
 
@@ -1234,7 +1277,7 @@ rb_library_source_add_location (RBLibrarySource *source, GtkWindow *win)
 		if (uri != NULL) {
 			GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri);
 			if (vfsuri != NULL) {
-				rhythmdb_add_uri_async (source->priv->db, uri);
+				rhythmdb_add_uri (source->priv->db, uri);
 				gnome_vfs_uri_unref (vfsuri);
 			} else
 				rb_debug ("invalid uri: \"%s\"", uri);
@@ -1256,7 +1299,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 	GList *list, *uri_list, *i;
 
 	rb_debug ("parsing uri list");
-	list = gnome_vfs_uri_list_parse (data->data);
+	list = gnome_vfs_uri_list_parse ((char *) data->data);
 
 	if (list == NULL)
 		return FALSE;
@@ -1277,7 +1320,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 		char *uri = i->data;
 
 		if (uri != NULL)
-			rhythmdb_add_uri_async (source->priv->db, uri);
+			rhythmdb_add_uri (source->priv->db, uri);
 
 		g_free (uri);
 	}
@@ -1289,7 +1332,7 @@ impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 static gboolean
 impl_show_popup (RBSource *source)
 {
-	rb_bonobo_show_popup (GTK_WIDGET (source), LIBRARY_SOURCE_POPUP_PATH);
+	_rb_source_show_popup (RB_SOURCE (source), "/LibrarySourcePopup");
 	return TRUE;
 }
 
@@ -1429,7 +1472,6 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 
 	/* Unlocked */
 	rb_debug ("preparing to read lock for query");
-	rhythmdb_read_lock (source->priv->db);
 
 	if (source->priv->active_query) {
 		rb_debug ("killing active query");
@@ -1465,7 +1507,6 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 			rb_property_view_set_model (source->priv->albums,
 						    source->priv->cached_albums_model);
 			rb_entry_view_poll_model (source->priv->songs);
-			rhythmdb_read_unlock (source->priv->db);
 			return;
 		} else if (source->priv->cached_all_query) {
 			rb_debug ("sorting mismatch, freeing cached query");
@@ -1574,3 +1615,16 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 	rhythmdb_query_free (query);
 	g_object_unref (G_OBJECT (query_model));
 }
+
+static void
+songs_view_drag_data_received_cb (GtkWidget *widget,
+				  GdkDragContext *dc,
+				  gint x, gint y,
+				  GtkSelectionData *selection_data,
+				  guint info, guint time,
+				  RBLibrarySource *source)
+{
+	rb_debug ("data dropped on the library source song view");
+	rb_source_receive_drag (RB_SOURCE (source), selection_data);
+}
+
