@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  arch-tag: Implementation of GtkTreeModel iface containing RBSource objects
  *
@@ -72,7 +73,7 @@ static gboolean rb_sourcelist_model_row_draggable (RbTreeDragSource *drag_source
 						   GList *path_list);
 
 
-static GtkListStoreClass *parent_class = NULL;
+static GtkTreeModelFilterClass *parent_class = NULL;
 
 static guint rb_sourcelist_model_signals[LAST_SIGNAL] = { 0 };
 
@@ -108,7 +109,7 @@ rb_sourcelist_model_get_type (void)
 			NULL
 		};
 
-		rb_sourcelist_model_type = g_type_register_static (GTK_TYPE_LIST_STORE, "RBSourceListModel",
+		rb_sourcelist_model_type = g_type_register_static (GTK_TYPE_TREE_MODEL_FILTER, "RBSourceListModel",
 								   &rb_sourcelist_model_info, 0);
 		g_type_add_interface_static (rb_sourcelist_model_type,
 					     RB_TYPE_TREE_DRAG_SOURCE,
@@ -157,7 +158,6 @@ rb_sourcelist_model_drag_dest_init (RbTreeDragDestIface *iface)
   iface->drag_data_received = rb_sourcelist_model_drag_data_received;
   iface->row_drop_possible = rb_sourcelist_model_row_drop_possible;
   iface->row_drop_position = rb_sourcelist_model_row_drop_position;
-
 }
 
 static void
@@ -166,7 +166,6 @@ rb_sourcelist_model_drag_source_init (RbTreeDragSourceIface *iface)
   iface->row_draggable = rb_sourcelist_model_row_draggable;
   iface->drag_data_get = rb_sourcelist_model_drag_data_get;
   iface->drag_data_delete = rb_sourcelist_model_drag_data_delete;
-
 }
 
 static void
@@ -186,19 +185,53 @@ rb_sourcelist_model_finalize (GObject *object)
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
+static gboolean
+rb_sourcelist_is_row_visible (GtkTreeModel *model,
+			      GtkTreeIter *iter,
+			      gpointer data)
+{
+	RBSource *source;
+
+	gtk_tree_model_get (GTK_TREE_MODEL (model), iter,
+			    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
+	
+	if (source != NULL) {
+		gboolean visible;	
+		g_object_get (G_OBJECT (source), "visibility", &visible, NULL);
+
+		return visible;
+	} else {
+		return FALSE;
+	}
+}
+
+
 GtkTreeModel *
 rb_sourcelist_model_new (void)
 {
 	RBSourceListModel *model;
-	GType *column_types = g_new (GType, 4);
-	column_types[0] = GDK_TYPE_PIXBUF;
-	column_types[1] = G_TYPE_STRING;
-	column_types[2] = G_TYPE_POINTER;
-	column_types[3] = PANGO_TYPE_ATTR_LIST;
+	GtkListStore *store;
+ 	GType *column_types = g_new (GType, RB_SOURCELIST_MODEL_N_COLUMNS);
+
+	column_types[RB_SOURCELIST_MODEL_COLUMN_PLAYING] = G_TYPE_BOOLEAN;
+	column_types[RB_SOURCELIST_MODEL_COLUMN_PIXBUF] = GDK_TYPE_PIXBUF;
+	column_types[RB_SOURCELIST_MODEL_COLUMN_NAME] = G_TYPE_STRING;
+	column_types[RB_SOURCELIST_MODEL_COLUMN_SOURCE] = G_TYPE_POINTER;
+	column_types[RB_SOURCELIST_MODEL_COLUMN_ATTRIBUTES] = PANGO_TYPE_ATTR_LIST;
+	store = gtk_list_store_newv (RB_SOURCELIST_MODEL_N_COLUMNS, 
+				     column_types);
+
+ 	model = RB_SOURCELIST_MODEL (g_object_new (RB_TYPE_SOURCELIST_MODEL, 
+						   "child-model", store,
+						   "virtual-root", NULL,
+						   NULL));
+
+	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (model), 
+						rb_sourcelist_is_row_visible, 
+						NULL, NULL);
 
-	model = RB_SOURCELIST_MODEL (g_object_new (RB_TYPE_SOURCELIST_MODEL, NULL));
-	gtk_list_store_set_column_types (GTK_LIST_STORE (model), 4, column_types);
 	g_free (column_types);
+
 	return GTK_TREE_MODEL (model);
 }
 
@@ -248,7 +281,7 @@ rb_sourcelist_model_drag_data_received (RbTreeDragDest *drag_dest,
 		if (!dest)
 			return FALSE;
 		
-		path_str = g_strndup (selection_data->data, selection_data->length);
+		path_str = g_strndup ((char *) selection_data->data, selection_data->length);
 	
 		path = gtk_tree_path_new_from_string (path_str);
 		gtk_tree_model_get_iter (GTK_TREE_MODEL (model),
@@ -306,7 +339,7 @@ path_is_droppable (RBSourceListModel *model,
 		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
 				    RB_SOURCELIST_MODEL_COLUMN_SOURCE, &source, -1);
 		
-		return rb_source_can_rename (source);
+		return rb_source_can_cut (source);
 
 	}
 	return FALSE;
@@ -323,10 +356,21 @@ rb_sourcelist_model_row_drop_position (RbTreeDragDest   *drag_dest,
 	if (g_list_find (targets, gdk_atom_intern ("text/uri-list", TRUE))) {
 		if (!dest_path)
 			return FALSE;
+		if (!path_is_droppable (RB_SOURCELIST_MODEL (model), dest_path))
+			return FALSE;
 		*pos = GTK_TREE_VIEW_DROP_INTO_OR_BEFORE;
 		return TRUE;
 	}
-	
+
+	if ((g_list_find (targets, gdk_atom_intern ("text/x-rhythmbox-artist", TRUE))
+	     || g_list_find (targets, gdk_atom_intern ("text/x-rhythmbox-album", TRUE))
+	     || g_list_find (targets, gdk_atom_intern ("text/x-rhythmbox-genre", TRUE)))
+	    && !g_list_find (targets, gdk_atom_intern ("application/x-rhythmbox-source", TRUE))) {
+		rb_debug ("genre, album, or artist type");
+		*pos = GTK_TREE_VIEW_DROP_AFTER;
+		return TRUE;
+	}
+
 	if (!g_list_find (targets, gdk_atom_intern ("application/x-rhythmbox-source", TRUE))) {
 		rb_debug ("unsupported type");
 		return FALSE;
@@ -396,7 +440,7 @@ rb_sourcelist_model_drag_data_get (RbTreeDragSource *drag_source,
 	path_str = gtk_tree_path_to_string (path);
 	gtk_selection_data_set (selection_data,
 				gdk_atom_intern ("application/x-rhythmbox-source", TRUE),
-				8, path_str,
+				8, (guchar *) path_str,
 				strlen (path_str));
 	g_free (path_str);
 	gtk_tree_path_free (path);
