@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  arch-tag: Implementation of widget to display RhythmDB entries
  *
@@ -24,6 +25,7 @@
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkiconfactory.h>
+#include <gtk/gtkstock.h>
 #include <gtk/gtktooltips.h>
 #include <gdk/gdkkeysyms.h>
 #include <config.h>
@@ -34,7 +36,6 @@
 #include <math.h>
 
 #include "rb-tree-dnd.h"
-#include "rb-tree-view-column.h"
 #include "rb-entry-view.h"
 #include "rb-dialog.h"
 #include "rb-debug.h"
@@ -45,7 +46,6 @@
 #include "rb-string-helpers.h"
 #include "rb-stock-icons.h"
 #include "rb-preferences.h"
-#include "rb-tree-view.h"
 #include "eel-gconf-extensions.h"
 
 static const GtkTargetEntry rb_entry_view_drag_types[] = {{  "text/uri-list", 0, 0 }};
@@ -105,9 +105,14 @@ static void rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 				   const char *path,
 				   double rating,
 				   RBEntryView *view);
+static void rb_entry_view_pixbuf_clicked_cb (RBEntryView *view,
+					     const char *path,
+					     RBCellRendererPixbuf *cellpixbuf);
 static gboolean rb_entry_view_button_press_cb (GtkTreeView *treeview,
 					      GdkEventButton *event,
 					      RBEntryView *view);
+static gboolean rb_entry_view_popup_menu_cb (GtkTreeView *treeview,
+					     RBEntryView *view);
 static void rb_entry_view_entry_is_visible (RBEntryView *view, RhythmDBEntry *entry,
 					    gboolean *realized, gboolean *visible,
 					    GtkTreeIter *iter);
@@ -135,19 +140,20 @@ struct RBEntryViewPrivate
 	RhythmDBQueryModel *playing_model;
 	RhythmDBEntry *playing_entry;
 	gboolean playing_entry_in_view;
-	GtkTreeIter playing_entry_iter;
 
 	gboolean is_drag_source;
 	gboolean is_drag_dest;
 
 	GdkPixbuf *playing_pixbuf;
 	GdkPixbuf *paused_pixbuf;
+	GdkPixbuf *error_pixbuf;
 	
 	char *sorting_key;
 	guint sorting_gconf_notification_id;
 	GList *clickable_columns;
 	GtkTreeViewColumn *sorting_column;
 	gint sorting_order;
+	char *sorting_column_name;
 	struct RBEntryViewReverseSortingData *reverse_sorting_data;
 	gboolean resorting;
 
@@ -201,6 +207,23 @@ static GObjectClass *parent_class = NULL;
 
 static guint rb_entry_view_signals[LAST_SIGNAL] = { 0 };
 
+static gboolean
+type_ahead_search_func (GtkTreeModel *model, gint column,
+			const gchar *key, GtkTreeIter *iter,
+			gpointer search_data)
+{
+	RhythmDBEntry *entry;
+	gchar *folded;
+	gboolean res;
+
+	gtk_tree_model_get (model, iter, 0, &entry, -1);
+	folded = g_utf8_casefold (key, -1);
+	res = (strstr (rb_refstring_get_folded (entry->title), folded) == NULL);
+	g_free (folded);
+	return res;
+}
+
+
 GType
 rb_entry_view_get_type (void)
 {
@@ -398,6 +421,10 @@ rb_entry_view_init (RBEntryView *view)
 							    RB_STOCK_PAUSED,
 							    GTK_ICON_SIZE_MENU,
 							    NULL);
+	view->priv->error_pixbuf = gtk_widget_render_icon (dummy,
+							   RB_STOCK_PLAYBACK_ERROR,
+							   GTK_ICON_SIZE_MENU,
+							   NULL);
 	gtk_widget_destroy (dummy);
 
 	view->priv->propid_column_map = g_hash_table_new (NULL, NULL);
@@ -433,6 +460,7 @@ rb_entry_view_finalize (GObject *object)
 
 	g_object_unref (G_OBJECT (view->priv->playing_pixbuf));
 	g_object_unref (G_OBJECT (view->priv->paused_pixbuf));
+	g_object_unref (G_OBJECT (view->priv->error_pixbuf));
 
 	g_free (view->priv->sorting_key);
 
@@ -504,18 +532,22 @@ rb_entry_view_set_property (GObject *object,
 			g_assert (sort_data);
 
 			if (view->priv->sorting_order != GTK_SORT_DESCENDING) {
-				g_object_set (G_OBJECT (new_model), "sort-func",
-					      sort_data->func, "sort-data", sort_data->data, NULL);
+				g_object_set (G_OBJECT (new_model),
+					      "sort-func", sort_data->func,
+					      "sort-data", sort_data->data,
+					      "sort-data-destroy", NULL,
+					      NULL);
 			} else {
-				g_free (view->priv->reverse_sorting_data);
 				view->priv->reverse_sorting_data
 					= g_new (struct RBEntryViewReverseSortingData, 1);
 				view->priv->reverse_sorting_data->func = sort_data->func;
 				view->priv->reverse_sorting_data->data = sort_data->data;
 				
-				g_object_set (G_OBJECT (new_model), "sort-func",
-					      reverse_sorting_func, "sort-data",
-					      view->priv->reverse_sorting_data, NULL);
+				g_object_set (G_OBJECT (new_model),
+					      "sort-func", reverse_sorting_func,
+					      "sort-data", view->priv->reverse_sorting_data,
+					      "sort-data-destroy", g_free,
+					      NULL);
 			}
 		}
 
@@ -547,10 +579,13 @@ rb_entry_view_set_property (GObject *object,
 		
 		if (view->priv->playing_entry != NULL
 		    && view->priv->playing_entry_in_view) {
+			rhythmdb_query_model_entry_to_iter (view->priv->playing_model,
+							    view->priv->playing_entry,
+							    &iter);
 			path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->playing_model),
-							&view->priv->playing_entry_iter);
+							&iter);
 			gtk_tree_model_row_changed (GTK_TREE_MODEL (view->priv->playing_model),
-						    path, &view->priv->playing_entry_iter);
+						    path, &iter);
 			gtk_tree_path_free (path);
 			g_object_unref (G_OBJECT (view->priv->playing_model));
 		}
@@ -563,12 +598,12 @@ rb_entry_view_set_property (GObject *object,
 			view->priv->playing_entry_in_view = 
 				rhythmdb_query_model_entry_to_iter (view->priv->model,
 								    view->priv->playing_entry,
-								    &view->priv->playing_entry_iter);
+								    &iter);
 			if (view->priv->playing_entry_in_view) {
 				path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->model),
-								&view->priv->playing_entry_iter);
+								&iter);
 				gtk_tree_model_row_changed (GTK_TREE_MODEL (view->priv->model),
-							    path, &view->priv->playing_entry_iter);
+							    path, &iter);
 				gtk_tree_path_free (path);
 			}
 		}
@@ -710,51 +745,35 @@ static gint
 rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			       RBEntryView *view)
 {
-	gint a_int, b_int;
-	const char *a_str = NULL;
-	const char *b_str = NULL;
 	gint ret;
 
 	/* Sort by album name */
-	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
-	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
-
-	ret = strcmp (a_str, b_str);
+	ret = strcmp (rb_refstring_get_sort_key (a->album),
+		      rb_refstring_get_sort_key (b->album));
 	if (ret != 0)
 		return ret;
 
 	/* Then by disc number, */
-	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_DISC_NUMBER);
-	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_DISC_NUMBER);
-	if (a_int != b_int)
-		return (a_int < b_int ? -1 : 1);
+	if (a->discnum != b->discnum)
+		return (a->discnum < b->discnum ? -1 : 1);
 
 	/* by track number */
-	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_TRACK_NUMBER);
-	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_TRACK_NUMBER);
-
-	if (a_int != b_int)
-		return (a_int < b_int ? -1 : 1);
+	if (a->tracknum != b->tracknum)
+		return (a->tracknum < b->tracknum ? -1 : 1);
 
 	/* And finally by title */
-	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_TITLE_SORT_KEY);
-	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_TITLE_SORT_KEY);
-
-	return strcmp (a_str, b_str);
+	return strcmp (rb_refstring_get_sort_key (a->title),
+		       rb_refstring_get_sort_key (b->title));
 }
 
 static gint
 rb_entry_view_artist_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 				RBEntryView *view)
 {
-	const char *a_str = NULL;
-	const char *b_str = NULL;
 	gint ret;
 
-	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_ARTIST_SORT_KEY);
-	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_ARTIST_SORT_KEY);
-
-	ret = strcmp (a_str, b_str);
+	ret = strcmp (rb_refstring_get_sort_key (a->artist),
+		      rb_refstring_get_sort_key (b->artist));
 	if (ret != 0)
 		return ret;
 
@@ -765,14 +784,10 @@ static gint
 rb_entry_view_genre_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			       RBEntryView *view)
 {
-	const char *a_str = NULL;
-	const char *b_str = NULL;
 	gint ret;
 
-	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_GENRE_SORT_KEY);
-	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_GENRE_SORT_KEY);
-
-	ret = strcmp (a_str, b_str);
+	ret = strcmp (rb_refstring_get_sort_key (a->genre),
+		      rb_refstring_get_sort_key (b->genre));
 	if (ret != 0)
 		return ret;
 
@@ -786,22 +801,6 @@ rb_entry_view_track_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	return rb_entry_view_album_sort_func (a, b, view);
 }
 
-
-static gint
-rb_entry_view_int_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-			     struct RBEntryViewCellDataFuncData *data)
-{
-	gint a_val, b_val;
-	gint ret;
-
-	a_val = rhythmdb_entry_get_int (data->view->priv->db, a, data->propid);
-	b_val = rhythmdb_entry_get_int (data->view->priv->db, b, data->propid);
-
-	ret = (a_val == b_val ? 0 : (a_val > b_val ? 1 : -1));
-
-	return ret;
-}
-
 static gint
 rb_entry_view_double_ceiling_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 				       struct RBEntryViewCellDataFuncData *data)
@@ -809,8 +808,8 @@ rb_entry_view_double_ceiling_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	gdouble a_val, b_val;
 	gint ret;
 
-	a_val = ceil (rhythmdb_entry_get_double (data->view->priv->db, a, data->propid));
-	b_val = ceil (rhythmdb_entry_get_double (data->view->priv->db, b, data->propid));
+	a_val = ceil (rhythmdb_entry_get_double (a, data->propid));
+	b_val = ceil (rhythmdb_entry_get_double (b, data->propid));
 
 	ret = (a_val == b_val ? 0 : (a_val > b_val ? 1 : -1));
 
@@ -818,14 +817,14 @@ rb_entry_view_double_ceiling_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 }
 
 static gint
-rb_entry_view_long_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-			      struct RBEntryViewCellDataFuncData *data)
+rb_entry_view_ulong_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+			       struct RBEntryViewCellDataFuncData *data)
 {
-	glong a_val, b_val;
+	gulong a_val, b_val;
 	gint ret;
 
-	a_val = rhythmdb_entry_get_long (data->view->priv->db, a, data->propid);
-	b_val = rhythmdb_entry_get_long (data->view->priv->db, b, data->propid);
+	a_val = rhythmdb_entry_get_ulong (a, data->propid);
+	b_val = rhythmdb_entry_get_ulong (b, data->propid);
 
 	ret = (a_val == b_val ? 0 : (a_val > b_val ? 1 : -1));
 
@@ -840,8 +839,8 @@ rb_entry_view_string_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	const char *b_val;	
 	gint ret;
 
-	a_val = rhythmdb_entry_get_string (data->view->priv->db, a, data->propid);
-	b_val = rhythmdb_entry_get_string (data->view->priv->db, b, data->propid);
+	a_val = rhythmdb_entry_get_string (a, data->propid);
+	b_val = rhythmdb_entry_get_string (b, data->propid);
 
 	ret = strcmp (a_val, b_val);
 
@@ -862,6 +861,8 @@ rb_entry_view_playing_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 		pixbuf = view->priv->playing_pixbuf;
 	else if (entry == view->priv->playing_entry)
 		pixbuf = view->priv->paused_pixbuf;
+	else if (entry->playback_error)
+		pixbuf = view->priv->error_pixbuf;
 	else
 		pixbuf = NULL;
 
@@ -874,37 +875,27 @@ rb_entry_view_rating_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 				     RBEntryView *view)
 {
 	RhythmDBEntry *entry;
-	gdouble rating;
 
 	entry = entry_from_tree_iter (view, iter);
 
-	rhythmdb_read_lock (view->priv->db);
-
-	rating = rhythmdb_entry_get_double (view->priv->db, entry, RHYTHMDB_PROP_RATING);
-
-	rhythmdb_read_unlock (view->priv->db);
-
-	g_object_set (G_OBJECT (renderer), "rating", rating, NULL);
+	g_object_set (G_OBJECT (renderer), "rating", entry->rating, NULL);
 }
 
 static void
-rb_entry_view_intstr_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-				  GtkTreeModel *tree_model, GtkTreeIter *iter,
-				  struct RBEntryViewCellDataFuncData *data)
+rb_entry_view_long_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
+				   GtkTreeModel *tree_model, GtkTreeIter *iter,
+				   struct RBEntryViewCellDataFuncData *data)
 {
 	RhythmDBEntry *entry;
 	char *str;
-	int val;
+	gulong val;
 
 	entry = entry_from_tree_iter (data->view, iter);
 
-	rhythmdb_read_lock (data->view->priv->db);
+	val = rhythmdb_entry_get_ulong (entry, data->propid);
 
-	val = rhythmdb_entry_get_int (data->view->priv->db, entry, data->propid);
-	rhythmdb_read_unlock (data->view->priv->db);
-
-	if (val >= 0)
-		str = g_strdup_printf ("%d", val);
+	if (val > 0)
+		str = g_strdup_printf ("%lu", val);
 	else
 		str = g_strdup ("");
 
@@ -914,24 +905,21 @@ rb_entry_view_intstr_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 
 static void
 rb_entry_view_play_count_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
-					 GtkTreeModel *tree_model, GtkTreeIter *iter,
+					 GtkTreeModel *tree_model, GtkTreeIter * iter,
 					 struct RBEntryViewCellDataFuncData *data)
 {
 	RhythmDBEntry *entry;
-	int i;
+	gulong i;
 	char *str;
 
 	entry = entry_from_tree_iter (data->view, iter);
 
-	rhythmdb_read_lock (data->view->priv->db);
-
-	i = rhythmdb_entry_get_int (data->view->priv->db, entry, data->propid);
+	i = rhythmdb_entry_get_ulong (entry, data->propid);
 	if (i == 0)
 		str = _("Never");
 	else
-		str = g_strdup_printf ("%d", i);
+		str = g_strdup_printf ("%ld", i);
 
-	rhythmdb_read_unlock (data->view->priv->db);
 
 	g_object_set (G_OBJECT (renderer), "text", str, NULL);
 	if (i != 0)
@@ -945,20 +933,12 @@ rb_entry_view_duration_cell_data_func (GtkTreeViewColumn *column, GtkCellRendere
 {
 	RhythmDBEntry *entry;
 	char *str;
-	long duration;
 	int minutes, seconds;
 
 	entry = entry_from_tree_iter (data->view, iter);
 
-	rhythmdb_read_lock (data->view->priv->db);
-
-	duration  = rhythmdb_entry_get_long (data->view->priv->db, entry,
-					     data->propid);
-
-	rhythmdb_read_unlock (data->view->priv->db);
-
-	minutes = duration / 60;
-	seconds = duration % 60;
+	minutes = entry->duration / 60;
+	seconds = entry->duration % 60;
 
 	if (minutes == 0 && seconds == 0)
 		str = g_strdup (_("Unknown"));
@@ -970,6 +950,7 @@ rb_entry_view_duration_cell_data_func (GtkTreeViewColumn *column, GtkCellRendere
 	g_free (str);
 }
 
+#if 0
 static void
 rb_entry_view_quality_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
 				      GtkTreeModel *tree_model, GtkTreeIter *iter,
@@ -980,13 +961,9 @@ rb_entry_view_quality_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 
 	entry = entry_from_tree_iter (data->view, iter);
 
-	rhythmdb_read_lock (data->view->priv->db);
-
 	bitrate = rhythmdb_entry_get_int (data->view->priv->db, entry,
 					  data->propid);
 
-	rhythmdb_read_unlock (data->view->priv->db);
-
 	if (bitrate <= 0) {
 		g_object_set (G_OBJECT (renderer), "text", _("Unknown"), NULL);
 	} else if (bitrate <= 80) {
@@ -1003,7 +980,7 @@ rb_entry_view_quality_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 		g_object_set (G_OBJECT (renderer), "text", _("Perfect"), NULL);
 	}
 }
-
+#endif
 
 static void
 rb_entry_view_string_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
@@ -1015,77 +992,113 @@ rb_entry_view_string_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 
 	entry = entry_from_tree_iter (data->view, iter);
 
-	rhythmdb_read_lock (data->view->priv->db);
-
-	str = rhythmdb_entry_get_string (data->view->priv->db, entry, data->propid);
+	str = rhythmdb_entry_get_string (entry, data->propid);
 
 	g_object_set (G_OBJECT (renderer), "text", str, NULL);
 
-	rhythmdb_read_unlock (data->view->priv->db);
 }
 
 static void
 rb_entry_view_sync_sorting (RBEntryView *view)
 {
-	GList *columns, *tem;
 	GtkTreeViewColumn *column;
-	char **strs;
 	gint direction;
-	char *sorttype;
+	const char *column_name;
 
-	if (!view->priv->sorting_key)
-		return;
-
-	sorttype = eel_gconf_get_string (view->priv->sorting_key);
-
-	if (!sorttype || !*sorttype) {
-		rb_debug ("no sorting data available in gconf!");
-		return;
-	}
-
-	if (!strchr (sorttype, ',')) {
-		g_warning ("malformed sort data");
-		return;
-	}
+	rb_entry_view_get_sorting_order (view, &column_name, &direction);
 	
-	strs = g_strsplit (sorttype, ",", 0);
+	if (!column_name)
+		return;
 
-	column = g_hash_table_lookup (view->priv->column_key_map, strs[0]);
+	column = g_hash_table_lookup (view->priv->column_key_map, column_name);
 	if (!column)
-		goto free_out;
+		return;
 
-	if (!strcmp ("ascending", strs[1]))
-		direction = GTK_SORT_ASCENDING;
-	else if (!strcmp ("descending", strs[1]))
-		direction = GTK_SORT_DESCENDING;		
-	else if (!strcmp ("none", strs[1]))
-		direction = -1;
-	else
-		goto free_out;
-
-  	columns = gtk_tree_view_get_columns (GTK_TREE_VIEW (view->priv->treeview));
-	for (tem = columns; tem; tem = tem->next)
-		gtk_tree_view_column_set_sort_indicator (tem->data, FALSE); 
-	g_list_free (columns);
+	rb_debug("Updating EntryView sort order to %s:%d", column_name, direction);
 	
+	/* remove the old sorting indicator */
+	if (view->priv->sorting_column)
+		gtk_tree_view_column_set_sort_indicator (view->priv->sorting_column, FALSE);
+	
+	/* set the sorting order and indicator of the new sorting column */
 	view->priv->sorting_column = column;
-	view->priv->sorting_order = direction;
+	gtk_tree_view_column_set_sort_indicator (column, TRUE);
+	gtk_tree_view_column_set_sort_order (column, direction);
 
-	if (view->priv->sorting_order != -1) {
-		gtk_tree_view_column_set_sort_indicator (column, TRUE);
-		gtk_tree_view_column_set_sort_order (column, view->priv->sorting_order);
-
-		rb_debug ("emitting sort order changed");
-		g_signal_emit (G_OBJECT (view), rb_entry_view_signals[SORT_ORDER_CHANGED], 0); 
-	}
-free_out:
-	g_strfreev (strs);
+	rb_debug ("emitting sort order changed");
+	g_signal_emit (G_OBJECT (view), rb_entry_view_signals[SORT_ORDER_CHANGED], 0); 
 }
 
 const char *
 rb_entry_view_get_sorting_type (RBEntryView *view)
 {
-	return eel_gconf_get_string (view->priv->sorting_key);
+	char *sorttype;
+	GString *key = g_string_new (view->priv->sorting_column_name);
+
+	g_string_append_c (key, ',');
+
+	switch (view->priv->sorting_order)
+	{
+	case GTK_SORT_ASCENDING:
+		g_string_append (key, "ascending");
+		break;
+	case GTK_SORT_DESCENDING:
+		g_string_append (key, "descending");
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+	
+	sorttype = g_strdup(key->str);
+	g_string_free (key, TRUE);
+	
+	return sorttype;
+}
+
+void
+rb_entry_view_set_sorting_type (RBEntryView *view, const char *sorttype)
+{
+	char **strs;
+
+	if (!sorttype || !strchr (sorttype, ',')) {
+		g_warning ("malformed sort data");
+		return;
+	}
+	
+	strs = g_strsplit (sorttype, ",", 0);
+	
+	g_free(view->priv->sorting_column_name);
+	view->priv->sorting_column_name = g_strdup(strs[0]);
+	
+	if (!strcmp ("ascending", strs[1]))
+		view->priv->sorting_order = GTK_SORT_ASCENDING;
+	else if (!strcmp ("descending", strs[1]))
+		view->priv->sorting_order = GTK_SORT_DESCENDING;		
+	else {
+		g_warning ("atttempting to sort in unknown direction");
+		view->priv->sorting_order = GTK_SORT_ASCENDING;
+	}
+	
+	g_strfreev (strs);
+	
+	rb_entry_view_sync_sorting (view);
+}
+
+void 
+rb_entry_view_get_sorting_order (RBEntryView *view, const char **column_name, gint *sort_order)
+{
+	*column_name = g_strdup(view->priv->sorting_column_name);
+	*sort_order = view->priv->sorting_order;
+}
+
+void
+rb_entry_view_set_sorting_order (RBEntryView *view, const char *column_name, gint sort_order)
+{
+	g_free(view->priv->sorting_column_name);
+	view->priv->sorting_column_name = g_strdup(column_name);
+	view->priv->sorting_order = sort_order;
+	
+	rb_entry_view_sync_sorting (view);
 }
 
 void
@@ -1099,41 +1112,27 @@ rb_entry_view_set_resorting (RBEntryView *view)
 static void
 rb_entry_view_column_clicked_cb (GtkTreeViewColumn *column, RBEntryView *view)
 {
-	GString *key = g_string_new ("");
-	gboolean is_sorted;
-	gint sorting_order;
+	gint sort_order;
+	char *clicked_column;
 
 	rb_debug ("sorting on column %p", column);
-	g_string_append (key, (char*) g_object_get_data (G_OBJECT (column), "rb-entry-view-key"));
-	g_string_append_c (key, ',');
-
-	is_sorted = gtk_tree_view_column_get_sort_indicator (column);
-
-	if (is_sorted) {
-		sorting_order = gtk_tree_view_column_get_sort_order (column);
-		if (sorting_order == GTK_SORT_ASCENDING)
-			sorting_order = GTK_SORT_DESCENDING;
-		else
-			sorting_order = -1;
-	} else
-		sorting_order = GTK_SORT_ASCENDING;
-
-	switch (sorting_order)
-	{
-	case -1:
-		g_string_append (key, "none");
-		break;
-	case GTK_SORT_ASCENDING:
-		g_string_append (key, "ascending");
-		break;
-	case GTK_SORT_DESCENDING:
-		g_string_append (key, "descending");
-		break;
-	default:
-		g_assert_not_reached ();
-	}
-	eel_gconf_set_string (view->priv->sorting_key, key->str);
-	g_string_free (key, TRUE);
+	
+	/* identify the clicked column, and then update the sorting order */
+	clicked_column = (char*) g_object_get_data (G_OBJECT (column), "rb-entry-view-key");
+	sort_order = view->priv->sorting_order;
+	
+	if (view->priv->sorting_column_name
+	    && !strcmp(clicked_column, view->priv->sorting_column_name)
+	    && (sort_order == GTK_SORT_ASCENDING))
+		sort_order = GTK_SORT_DESCENDING;
+	else
+		sort_order = GTK_SORT_ASCENDING;
+	
+	rb_entry_view_set_sorting_order (view, clicked_column, sort_order);
+	
+	/* update the sort order in GConf */
+	if (view->priv->sorting_key)
+		eel_gconf_set_string (view->priv->sorting_key, rb_entry_view_get_sorting_type(view));
 }
 
 void
@@ -1149,11 +1148,12 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 	GCompareDataFunc sort_func = NULL;
 	gpointer real_sort_data = NULL;
 	RhythmDBPropType propid;
+	gboolean ellipsize = FALSE;
 
-	column = GTK_TREE_VIEW_COLUMN (rb_tree_view_column_new ());
+	column = gtk_tree_view_column_new ();
 
 	if (coltype == RB_ENTRY_VIEW_COL_RATING) {
-		guint width;
+		gint width;
 
 		propid = RHYTHMDB_PROP_RATING;
 
@@ -1194,7 +1194,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 	case RB_ENTRY_VIEW_COL_TRACK_NUMBER:
 		propid = RHYTHMDB_PROP_TRACK_NUMBER;
 		cell_data->propid = propid;
-		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_intstr_cell_data_func;
+		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_long_cell_data_func;
 		sort_func = (GCompareDataFunc) rb_entry_view_track_sort_func;
 		real_sort_data = view;
 		title = _("Tra_ck");
@@ -1208,7 +1208,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		sort_func = (GCompareDataFunc) rb_entry_view_string_sort_func;
 		title = _("_Title");
 		key = "Title";
-		rb_tree_view_column_set_expand (RB_TREE_VIEW_COLUMN (column), TRUE);
+		ellipsize = TRUE;
 		break;
 	case RB_ENTRY_VIEW_COL_ARTIST:
 		propid = RHYTHMDB_PROP_ARTIST;
@@ -1219,7 +1219,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		real_sort_data = view;
 		title = _("Art_ist");
 		key = "Artist";
-		rb_tree_view_column_set_expand (RB_TREE_VIEW_COLUMN (column), TRUE);
+		ellipsize = TRUE;
 		break;
 	case RB_ENTRY_VIEW_COL_ALBUM:
 		propid = RHYTHMDB_PROP_ALBUM;
@@ -1230,7 +1230,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		real_sort_data = view;
 		title = _("A_lbum");
 		key = "Album";
-		rb_tree_view_column_set_expand (RB_TREE_VIEW_COLUMN (column), TRUE);
+		ellipsize = TRUE;
 		break;
 	case RB_ENTRY_VIEW_COL_GENRE:
 		propid = RHYTHMDB_PROP_GENRE;
@@ -1241,33 +1241,35 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		real_sort_data = view;
 		title = _("Ge_nre");
 		key = "Genre";
-		rb_tree_view_column_set_expand (RB_TREE_VIEW_COLUMN (column), TRUE);
+		ellipsize = TRUE;
 		break;
 	case RB_ENTRY_VIEW_COL_DURATION:
 		propid = RHYTHMDB_PROP_DURATION;
 		cell_data->propid = propid;
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_duration_cell_data_func;
 		sort_data->propid = cell_data->propid;
-		sort_func = (GCompareDataFunc) rb_entry_view_long_sort_func;
+		sort_func = (GCompareDataFunc) rb_entry_view_ulong_sort_func;
 		title = _("Ti_me");
 		key = "Time";
 		break;
+#if 0
 	case RB_ENTRY_VIEW_COL_QUALITY:
 		propid = RHYTHMDB_PROP_BITRATE;
 		cell_data->propid = propid;
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_quality_cell_data_func;
 		sort_data->propid = cell_data->propid;
-		sort_func = (GCompareDataFunc) rb_entry_view_int_sort_func;
+		sort_func = (GCompareDataFunc) rb_entry_view_ulong_sort_func;
 		title = _("_Quality");
 		key = "Quality";
 		break;
+#endif
 	/* RB_ENTRY_VIEW_COL_RATING at bottom */
 	case RB_ENTRY_VIEW_COL_PLAY_COUNT:
 		propid = RHYTHMDB_PROP_PLAY_COUNT;
 		cell_data->propid = propid;
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_play_count_cell_data_func;
 		sort_data->propid = cell_data->propid;
-		sort_func = (GCompareDataFunc) rb_entry_view_int_sort_func;
+		sort_func = (GCompareDataFunc) rb_entry_view_ulong_sort_func;
 		title = _("_Play Count");
 		key = "PlayCount";
 		break;
@@ -1276,7 +1278,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		cell_data->propid = RHYTHMDB_PROP_LAST_PLAYED_STR;
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;				
 		sort_data->propid = RHYTHMDB_PROP_LAST_PLAYED;
-		sort_func = (GCompareDataFunc) rb_entry_view_long_sort_func;
+		sort_func = (GCompareDataFunc) rb_entry_view_ulong_sort_func;
 		title = _("L_ast Played");
 		key = "LastPlayed";
 		break;
@@ -1288,6 +1290,10 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 	}
 
 	renderer = gtk_cell_renderer_text_new ();
+	if (ellipsize) {
+		g_object_set (G_OBJECT (renderer), "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+		gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
+	}
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_set_cell_data_func (column, renderer,
 						 cell_data_func, cell_data, g_free);
@@ -1361,7 +1367,11 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 	view = RB_ENTRY_VIEW (parent_class->constructor (type, n_construct_properties,
 							 construct_properties));
 
-	view->priv->treeview = GTK_WIDGET (rb_tree_view_new ());
+	view->priv->treeview = GTK_WIDGET (gtk_tree_view_new ());
+
+	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (view->priv->treeview),
+					     type_ahead_search_func, 
+					     NULL, NULL);
 
 	g_signal_connect_object (G_OBJECT (view->priv->treeview),
 			         "button_press_event",
@@ -1373,6 +1383,11 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 			         G_CALLBACK (rb_entry_view_row_activated_cb),
 			         view,
 				 0);
+	g_signal_connect_object (G_OBJECT (view->priv->treeview),
+				 "popup_menu",
+				 G_CALLBACK (rb_entry_view_popup_menu_cb),
+				 view,
+				 0);
 	view->priv->selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view->priv->treeview));
 	g_signal_connect_object (G_OBJECT (view->priv->selection),
 			         "changed",
@@ -1403,12 +1418,12 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 		GtkTreeViewColumn *column;
 		GtkTooltips *tooltip;
 		GtkCellRenderer *renderer;
-		guint width;
+		gint width;
 
 		tooltip = gtk_tooltips_new ();
 		
 		/* Playing icon column */
-		column = GTK_TREE_VIEW_COLUMN (rb_tree_view_column_new ());
+		column = GTK_TREE_VIEW_COLUMN (gtk_tree_view_column_new ());
 		renderer = rb_cell_renderer_pixbuf_new ();
 		gtk_tree_view_column_pack_start (column, renderer, TRUE);
 		gtk_tree_view_column_set_cell_data_func (column, renderer,
@@ -1420,6 +1435,10 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 		gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, NULL);
 		gtk_tree_view_column_set_fixed_width (column, width + 5);
 		gtk_tree_view_append_column (GTK_TREE_VIEW (view->priv->treeview), column);
+		g_signal_connect_swapped (renderer,
+					  "pixbuf-clicked",
+					  G_CALLBACK (rb_entry_view_pixbuf_clicked_cb),
+					  G_OBJECT (view));
 
 		gtk_tooltips_set_tip (GTK_TOOLTIPS (tooltip), GTK_WIDGET (column->button),
 				       _("Now Playing"), NULL);
@@ -1435,6 +1454,9 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 						    rb_entry_view_sort_key_changed_cb,
 						    view);
 	
+	if (view->priv->sorting_key)
+		rb_entry_view_set_sorting_type (view, eel_gconf_get_string (view->priv->sorting_key));
+	
 	{
 		RhythmDBQueryModel *query_model;
 		query_model = rhythmdb_query_model_new_empty (view->priv->db);
@@ -1477,16 +1499,39 @@ rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 
 	g_value_init (&value, G_TYPE_DOUBLE);
 	g_value_set_double (&value, rating);
-	rhythmdb_entry_queue_set (view->priv->db, entry, RHYTHMDB_PROP_RATING,
-				  &value);
+	rhythmdb_entry_sync (view->priv->db, entry, RHYTHMDB_PROP_RATING,
+			    &value);
 	g_value_unset (&value);
 
 	/* since the user changed the rating, stop auto-rating */
 	g_value_init (&value, G_TYPE_BOOLEAN);
 	g_value_set_boolean (&value, FALSE);
-	rhythmdb_entry_queue_set (view->priv->db, entry, RHYTHMDB_PROP_AUTO_RATE,
-				  &value);
+	rhythmdb_entry_sync (view->priv->db, entry, RHYTHMDB_PROP_AUTO_RATE,
+			    &value);
 	g_value_unset (&value);
+
+	rhythmdb_commit (view->priv->db);
+}
+
+static void
+rb_entry_view_pixbuf_clicked_cb (RBEntryView          *view,
+				 const char           *path_string,
+				 RBCellRendererPixbuf *cellpixbuf)
+{
+	GtkTreePath *path;
+	RhythmDBEntry *entry;
+
+	g_return_if_fail (path_string != NULL);
+
+	path = gtk_tree_path_new_from_string (path_string);
+	entry = entry_from_tree_path (view, path);
+
+	gtk_tree_path_free (path);
+
+	if (entry->playback_error) {
+		rb_error_dialog (NULL, _("Playback Error"),
+				 "%s", entry->playback_error);
+	}
 }
 
 void
@@ -1669,6 +1714,18 @@ rb_entry_view_button_press_cb (GtkTreeView *treeview,
 	return FALSE;
 }
 
+static gboolean
+rb_entry_view_popup_menu_cb (GtkTreeView *treeview,
+			     RBEntryView *view)
+{
+	if (gtk_tree_selection_count_selected_rows (gtk_tree_view_get_selection (treeview)) == 0)
+		return FALSE;
+
+	g_signal_emit (G_OBJECT (view), rb_entry_view_signals[SHOW_POPUP], 0);
+	return TRUE;
+}
+
+
 static void
 queue_changed_sig (RBEntryView *view)
 {
@@ -1954,7 +2011,7 @@ rb_entry_view_sort_key_changed_cb (GConfClient* client,
 
 	g_return_if_fail (RB_IS_ENTRY_VIEW (view));
 
-	rb_entry_view_sync_sorting (view);
+	rb_entry_view_set_sorting_type (view, eel_gconf_get_string (view->priv->sorting_key));
 }
 
 static void
@@ -1974,13 +2031,10 @@ static gint
 propid_from_name (const char *name)
 {
 	GEnumClass *prop_class = g_type_class_ref (RHYTHMDB_TYPE_PROP);
-	GEnumClass *unsaved_prop_class = g_type_class_ref (RHYTHMDB_TYPE_UNSAVED_PROP);
 	GEnumValue *ev;
 	int ret;
 
 	ev = g_enum_get_value_by_name (prop_class, name);
-	if (!ev)
-		ev = g_enum_get_value_by_name (unsaved_prop_class, name);
 	if (ev)
 		ret = ev->value;
 	else
@@ -2052,4 +2106,3 @@ rb_entry_view_poll_model (RBEntryView *view)
 
 	return rhythmdb_query_model_poll (view->priv->model, &timeout);
 }
-
