@@ -20,6 +20,7 @@
  *
  */
 
+#include <math.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
@@ -36,7 +37,6 @@
 #include "rb-stock-icons.h"
 #include "rb-header.h"
 #include "rb-debug.h"
-#include "rb-ellipsizing-label.h"
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
 
@@ -54,13 +54,14 @@ static void rb_header_get_property (GObject *object,
 static long rb_header_get_duration (RBHeader *player);
 static void rb_header_set_show_timeline (RBHeader *player,
 			                 gboolean show);
-static gboolean rb_header_sync_time_locked (RBHeader *player);
 static void rb_header_update_elapsed (RBHeader *player);
 static gboolean slider_press_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *player);
 static gboolean slider_moved_callback (GtkWidget *widget, GdkEventMotion *event, RBHeader *player);
 static gboolean slider_release_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *player);
 static void slider_changed_callback (GtkWidget *widget, RBHeader *player);
 
+static void rb_header_tick_cb (RBPlayer *mmplayer, long time, RBHeader *header);
+
 typedef struct
 {
 	long elapsed;
@@ -76,6 +77,7 @@ struct RBHeaderPrivate
 	char *urltext, *urllink;
 
 	RBPlayer *mmplayer;
+	gulong tick_id;
 
 	GtkWidget *image;
 	GtkWidget *song;
@@ -101,8 +103,6 @@ struct RBHeaderPrivate
 	gboolean urlline_shown;
 	GnomeHRef *url;
 
-	guint timeout;
-	
 	RBHeaderState *state;
 };
 
@@ -110,7 +110,7 @@ enum
 {
 	PROP_0,
 	PROP_DB,
-	PROP_HEADER,
+	PROP_PLAYER,
 	PROP_ENTRY,
 	PROP_TITLE,
 	PROP_URLTEXT,
@@ -121,9 +121,6 @@ static GObjectClass *parent_class = NULL;
 
 #define SONG_MARKUP(xSONG) g_strdup_printf ("<big><b>%s</b></big>", xSONG);
 
-#define ALBUM_INFO_URL(xALBUM)   g_strdup_printf ("http://www.allmusic.com/cg/amg.dll?p=amg&opt1=2&sql=%s", xALBUM);
-#define ARTIST_INFO_URL(xARTIST) g_strdup_printf ("http://www.allmusic.com/cg/amg.dll?p=amg&opt1=1&sql=%s", xARTIST);
-
 GType
 rb_header_get_type (void)
 {
@@ -178,10 +175,10 @@ rb_header_class_init (RBHeaderClass *klass)
 							       "RhythmDBEntry pointer",
 							       G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
-					 PROP_HEADER,
+					 PROP_PLAYER,
 					 g_param_spec_object ("player",
 							      "Player",
-							      "MonkeyMediaPlayer",
+							      "RBPlayer object",
 							      RB_TYPE_PLAYER,
 							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
@@ -218,7 +215,7 @@ rb_header_init (RBHeader *player)
 	 *     GtkVBox
 	 *       GtkAlignment
 	 *         GtkVBox
-	 *           RBEllipsizingLabel	(priv->song)
+	 *           GtkLabel		(priv->song)
 	 *           GtkHBox		(priv->textframe)
 	 *	       RBSongDisplayBox (priv->displaybox)
 	 *               GtkLabel	("from")
@@ -255,8 +252,8 @@ rb_header_init (RBHeader *player)
 	textvbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (align), textvbox);
 	
-	player->priv->song = rb_ellipsizing_label_new ("");
-	rb_ellipsizing_label_set_mode (RB_ELLIPSIZING_LABEL (player->priv->song), RB_ELLIPSIZE_END);
+	player->priv->song = gtk_label_new ("");
+	gtk_label_set_ellipsize (GTK_LABEL (player->priv->song), PANGO_ELLIPSIZE_END);
  	gtk_label_set_use_markup (GTK_LABEL (player->priv->song), TRUE);
  	gtk_label_set_selectable (GTK_LABEL (player->priv->song), TRUE);	
 	gtk_misc_set_alignment (GTK_MISC (player->priv->song), 0, 0);
@@ -287,7 +284,7 @@ rb_header_init (RBHeader *player)
 	scalebox = player->priv->timeline = gtk_vbox_new (FALSE, 0);
 	g_object_ref (G_OBJECT (scalebox));
 
-	player->priv->adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 1.0, 0.01, 0.1, 0.0));
+	player->priv->adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 10.0, 0.0));
 	player->priv->scale = gtk_hscale_new (player->priv->adjustment);
 	g_signal_connect_object (G_OBJECT (player->priv->scale),
 				 "button_press_event",
@@ -320,8 +317,6 @@ rb_header_init (RBHeader *player)
 
 	gtk_box_pack_start (GTK_BOX (player), hbox, TRUE, TRUE, 0);
 	gtk_box_pack_end (GTK_BOX (player), align, FALSE, FALSE, 0);
-
-	player->priv->timeout = g_timeout_add (1000, (GSourceFunc) rb_header_sync_time_locked, player);
 }
 
 static void
@@ -336,8 +331,6 @@ rb_header_finalize (GObject *object)
 
 	g_return_if_fail (player->priv != NULL);
 
-	g_source_remove (player->priv->timeout);
-
 	g_object_unref (G_OBJECT (player->priv->urlline));
 	g_object_unref (G_OBJECT (player->priv->displaybox));
 	g_object_unref (G_OBJECT (player->priv->timeline));
@@ -363,10 +356,20 @@ rb_header_set_property (GObject *object,
 	case PROP_ENTRY:
 		player->priv->entry = g_value_get_pointer (value);
 		break;
-	case PROP_HEADER:
+	case PROP_PLAYER:
+		if (player->priv->mmplayer) {
+			g_signal_handler_disconnect (player->priv->mmplayer,
+						     player->priv->tick_id);
+			player->priv->tick_id = 0;
+		}
 		player->priv->mmplayer = g_value_get_object (value);
+		player->priv->tick_id = g_signal_connect (G_OBJECT (player->priv->mmplayer),
+							  "tick", 
+							  (GCallback) rb_header_tick_cb,
+							  player);
 		break;
 	case PROP_TITLE:
+		g_free (player->priv->title);
 		player->priv->title = g_value_dup_string (value);
 		break;
 	case PROP_URLTEXT:
@@ -398,7 +401,7 @@ rb_header_get_property (GObject *object,
 	case PROP_ENTRY:
 		g_value_set_object (value, player->priv->entry);
 		break;
-	case PROP_HEADER:
+	case PROP_PLAYER:
 		g_value_set_object (value, player->priv->mmplayer);
 		break;
 	case PROP_TITLE:
@@ -445,18 +448,70 @@ rb_header_set_title (RBHeader *player, const char *title)
 static long
 rb_header_get_duration (RBHeader *player)
 {
-	long ret;
 	if (player->priv->entry) {
-		rhythmdb_read_lock (player->priv->db);
-		ret = rhythmdb_entry_get_long (player->priv->db,
-					       player->priv->entry,
-					       RHYTHMDB_PROP_DURATION);
-		rhythmdb_read_unlock (player->priv->db);
-		return ret;
+		return player->priv->entry->duration;
 	}
 	return -1;
 }
 
+static char *
+info_url (guint opt, const char *data)
+{
+	char *ret, *d;
+
+	/*
+	 * netlabels often put URLs (or domain names) in the 'album' field
+	 * of their releases; since there are no artist names in AMG that
+	 * start with 'http://' or 'www.' (there is a 'www', from the 70s,
+	 * strangely enough), we can safely assume anything that looks 
+	 * like a URL or domain name is one.
+	 *
+	 * There's sometimes some trailing junk, usually after a space,
+	 * so it's probably sensible to strip that off.
+	 */
+	if (g_str_has_prefix (data, "http://")) {
+		char *end = strchr (data, ' ');
+		if (end != NULL)
+			ret = g_strndup (data, end - data);
+		else
+			ret = g_strdup (data);
+	} else if (g_str_has_prefix (data, "www.")) {
+		char *end = strchr (data, ' ');
+		if (end != NULL)
+			ret = g_strdup_printf ("http://%*s",
+					       (int) (end-data), data);
+		else
+			ret = g_strdup_printf ("http://%s", data);
+	} else {
+		char *r;
+
+		d = g_strdup (data);
+		r = d;
+		while (*r != '\0') {
+			if (*r == ' ')
+				*r = '+';
+			r++;
+		}
+
+		ret = g_strdup_printf ("http://www.allmusic.com/cg/amg.dll?p=amg&opt1=%d&sql=%s", opt, d);
+		g_free (d);
+	}
+
+	return ret;
+}
+
+static char *
+album_info_url (const char *album)
+{
+	return info_url (2, album);
+}
+
+static char *
+artist_info_url (const char *album)
+{
+	return info_url (1, album);
+}
+
 void
 rb_header_sync (RBHeader *player)
 {
@@ -465,28 +520,20 @@ rb_header_sync (RBHeader *player)
 	rb_debug ("syncing with node = %p", player->priv->entry);
 	if (player->priv->entry != NULL) {
 		const char *song = player->priv->title;
-		char *escaped, *s;
+		char *escaped;
 		gboolean have_duration = rb_header_get_duration (player) > 0;
 		const char *album; 
 		const char *artist; 
 		GtkTooltips *artist_href_tips, *album_href_tips;
 
-		rhythmdb_read_lock (player->priv->db);
 
-		album = rhythmdb_entry_get_string (player->priv->db,
-						   player->priv->entry,
-						   RHYTHMDB_PROP_ALBUM);
-
-		artist = rhythmdb_entry_get_string (player->priv->db,
-						    player->priv->entry,
-						    RHYTHMDB_PROP_ARTIST);
-
-		rhythmdb_read_unlock (player->priv->db);
+		album = rb_refstring_get (player->priv->entry->album);
+		artist = rb_refstring_get (player->priv->entry->artist);
 
 		escaped = g_markup_escape_text (song, -1);
 		tmp = SONG_MARKUP (escaped);
 		g_free (escaped);
-		rb_ellipsizing_label_set_markup (RB_ELLIPSIZING_LABEL (player->priv->song), tmp);
+		gtk_label_set_markup (GTK_LABEL (player->priv->song), tmp);
 		g_free (tmp);
 
 		rb_header_set_show_artist_album (player, (album != NULL && artist != NULL)
@@ -499,12 +546,7 @@ rb_header_sync (RBHeader *player)
 
 			rb_debug ("artist: %s album: %s", artist, album);
 
-			s = tmp = g_strdup (album);
-			while ((tmp = strstr (tmp, " ")) != NULL)
-				*tmp = '|';
-			tmp = ALBUM_INFO_URL (s);
-			g_free (s);
-
+			tmp = album_info_url (album);
 			gnome_href_set_url (player->priv->displaybox->album, tmp);
 			escaped = g_markup_escape_text (album, -1);
 			gnome_href_set_text (player->priv->displaybox->album, escaped);
@@ -517,13 +559,7 @@ rb_header_sync (RBHeader *player)
 					      _("Get information on this album from the web"), 
 					      NULL);
 			
-			s = tmp = g_strdup (artist);
-			while ((tmp = strstr (tmp, " ")) != NULL)
-			{
-				*tmp = '|';
-			}
-			tmp = ARTIST_INFO_URL (s);
-			g_free (s);
+			tmp = artist_info_url (artist);
 			gnome_href_set_url (player->priv->displaybox->artist, tmp);
 			escaped = g_markup_escape_text (artist, -1);
 			gnome_href_set_text (player->priv->displaybox->artist, escaped);
@@ -554,8 +590,8 @@ rb_header_sync (RBHeader *player)
 		GtkTooltips *iradio_href_tips;
 
 		rb_debug ("not playing");
-		tmp = SONG_MARKUP (_("Not playing"));
-		rb_ellipsizing_label_set_markup (RB_ELLIPSIZING_LABEL (player->priv->song), tmp);
+		tmp = SONG_MARKUP (_("Not Playing"));
+		gtk_label_set_markup (GTK_LABEL (player->priv->song), tmp);
 		g_free (tmp);
 
 		iradio_href_tips = gtk_tooltips_new ();
@@ -633,20 +669,6 @@ rb_header_set_show_timeline (RBHeader *player,
 	}
 }
 
-gboolean
-rb_header_sync_time_locked (RBHeader *player)
-{
-	gboolean ret;
-
-	gdk_threads_enter ();
-
-	ret = rb_header_sync_time (player);
-
-	gdk_threads_leave ();
-
-	return ret;
-}
-
 gboolean
 rb_header_sync_time (RBHeader *player)
 {
@@ -656,19 +678,24 @@ rb_header_sync_time (RBHeader *player)
 	if (player->priv->mmplayer == NULL)
 		return TRUE;
 
-	if (player->priv->slider_dragging == TRUE)
+	if (player->priv->slider_dragging == TRUE) {
+		rb_debug ("slider is dragging, not syncing");
 		return TRUE;
+	}
 
 	player->priv->state->duration = duration =
 		rb_header_get_duration (player);
-	player->priv->state->elapsed = seconds =
-		rb_player_get_time (player->priv->mmplayer);
+	seconds = player->priv->state->elapsed;
 
 	if (duration > -1) {
 		double progress = 0.0;
 
 		if (seconds > 0)
-			progress = (double) ((long) seconds) / duration;
+			progress = (double) (long) seconds;
+		else {
+			player->priv->adjustment->upper = duration;
+			g_signal_emit_by_name (G_OBJECT (player->priv->adjustment), "changed");
+		}
 
 		player->priv->slider_locked = TRUE;
 		gtk_adjustment_set_value (player->priv->adjustment, progress);
@@ -700,20 +727,20 @@ static gboolean
 slider_moved_timeout (RBHeader *player)
 {
 	double progress;
-	long duration, new;
+	long new;
 
-	gdk_threads_enter ();
+	GDK_THREADS_ENTER ();
 	
 	progress = gtk_adjustment_get_value (gtk_range_get_adjustment (GTK_RANGE (player->priv->scale)));
-	duration = rb_header_get_duration (player);
-	new = (long) (progress * duration);
+	new = (long) (progress+0.5);
 	
+	rb_debug ("setting time to %ld", new);
 	rb_player_set_time (player->priv->mmplayer, new);
 
 	player->priv->latest_set_time = new;
 	player->priv->slider_moved_timeout = 0;
 	
-	gdk_threads_leave ();
+	GDK_THREADS_LEAVE ();
 
 	return FALSE;
 }
@@ -725,21 +752,21 @@ slider_moved_callback (GtkWidget *widget,
 {
 	GtkAdjustment *adjustment;
 	double progress;
-	long duration;
 
-	if (player->priv->slider_dragging == FALSE)
+	if (player->priv->slider_dragging == FALSE) {
+		rb_debug ("slider is not dragging");
 		return FALSE;
+	}
 
 	adjustment = gtk_range_get_adjustment (GTK_RANGE (widget));
 
 	progress = gtk_adjustment_get_value (adjustment);
-	duration = rb_header_get_duration (player);
-
-	player->priv->state->elapsed = (long) (progress * duration);
+	player->priv->state->elapsed = (long) (progress+0.5);
 
 	rb_header_update_elapsed (player);
 
 	if (player->priv->slider_moved_timeout != 0) {
+		rb_debug ("removing old timer");
 		g_source_remove (player->priv->slider_moved_timeout);
 		player->priv->slider_moved_timeout = 0;
 	}
@@ -755,20 +782,23 @@ slider_release_callback (GtkWidget *widget,
 			 RBHeader *player)
 {
 	double progress;
-	long duration, new;
+	long new;
 	GtkAdjustment *adjustment;
 
-	if (player->priv->slider_dragging == FALSE)
+	if (player->priv->slider_dragging == FALSE) {
+		rb_debug ("slider is not dragging");
 		return FALSE;
+	}
 
 	adjustment = gtk_range_get_adjustment (GTK_RANGE (widget));
 
 	progress = gtk_adjustment_get_value (adjustment);
-	duration = rb_header_get_duration (player);
-	new = (long) (progress * duration);
+	new = (long) (progress+0.5);
 
-	if (new != player->priv->latest_set_time)
+	if (new != player->priv->latest_set_time) {
+		rb_debug ("setting time to %ld", new);
 		rb_player_set_time (player->priv->mmplayer, new);
+	}
 
 	player->priv->slider_dragging = FALSE;
 
@@ -786,14 +816,15 @@ static gboolean
 changed_idle_callback (RBHeader *player)
 {
 
-	gdk_threads_enter ();
+	GDK_THREADS_ENTER ();
 
 	slider_release_callback (player->priv->scale, NULL, player);
 
 	player->priv->value_changed_update_handler = 0;
 	rb_debug ("in changed_idle_callback"); 
 
-	gdk_threads_leave ();
+	GDK_THREADS_LEAVE ();
+
 	return FALSE;
 }
 
@@ -814,11 +845,10 @@ char *
 rb_header_get_elapsed_string (RBHeader *player)
 {
 	int seconds = 0, minutes = 0, seconds2 = -1, minutes2 = -1;
-	guint elapsed = rb_player_get_time (player->priv->mmplayer);
 
-	if (elapsed > 0) {
-		minutes = elapsed / 60;
-		seconds = elapsed % 60;
+	if (player->priv->state->elapsed > 0) {
+		minutes = player->priv->state->elapsed / 60;
+		seconds = player->priv->state->elapsed % 60;
 	}
 
 	if (player->priv->state->duration > 0) {
@@ -827,7 +857,7 @@ rb_header_get_elapsed_string (RBHeader *player)
 		if (eel_gconf_get_boolean (CONF_UI_TIME_DISPLAY)) {
 			return g_strdup_printf (_("%d:%02d of %d:%02d"), minutes, seconds, minutes2, seconds2);
 		} else {
-			int remaining = player->priv->state->duration - elapsed;
+			int remaining = player->priv->state->duration - player->priv->state->elapsed;
 			int remaining_minutes = remaining / 60;
 			/* remaining could conceivably be negative. This would
 			 * be a bug, but the elapsed time will display right
@@ -854,3 +884,17 @@ rb_header_update_elapsed (RBHeader *player)
 	gtk_label_set_text (GTK_LABEL (player->priv->elapsed), elapsed_text);
 	g_free (elapsed_text);
 }
+
+static void rb_header_tick_cb (RBPlayer *mmplayer, long elapsed, RBHeader *header)
+{
+	gboolean sync = (elapsed != header->priv->state->elapsed);
+	header->priv->state->elapsed = elapsed;
+	
+	if (sync) {
+		GDK_THREADS_ENTER ();
+
+		rb_header_sync_time (header);
+
+		GDK_THREADS_LEAVE ();
+	}
+}
