@@ -54,7 +54,8 @@ const RBQueryCreatorPropertyOption property_options[] =
 	{ N_("Artist"), RHYTHMDB_PROP_ARTIST, RHYTHMDB_PROP_ARTIST_FOLDED },
 	{ N_("Album"), RHYTHMDB_PROP_ALBUM, RHYTHMDB_PROP_ALBUM_FOLDED },
 	{ N_("Genre"), RHYTHMDB_PROP_GENRE, RHYTHMDB_PROP_GENRE_FOLDED },
-	{ N_("Rating"), RHYTHMDB_PROP_RATING, RHYTHMDB_PROP_RATING }
+	{ N_("Rating"), RHYTHMDB_PROP_RATING, RHYTHMDB_PROP_RATING },
+	{ N_("Path"), RHYTHMDB_PROP_LOCATION, RHYTHMDB_PROP_LOCATION }
 };
 
 const RBQueryCreatorCriteriaOption string_criteria_options[] =
@@ -67,8 +68,8 @@ const RBQueryCreatorCriteriaOption string_criteria_options[] =
 const RBQueryCreatorCriteriaOption numeric_criteria_options[] =
 {
 	{ N_("equals"), 1, RHYTHMDB_QUERY_PROP_EQUALS },
-	{ N_("greater than"), 1, RHYTHMDB_QUERY_PROP_GREATER },
-	{ N_("less than"), 1, RHYTHMDB_QUERY_PROP_LESS }
+	{ N_("at least"), 1, RHYTHMDB_QUERY_PROP_GREATER },	/* matches if A >= B */
+	{ N_("at most"), 1, RHYTHMDB_QUERY_PROP_LESS }		/* matches if A <= B */
 };
 
 static void rb_query_creator_class_init (RBQueryCreatorClass *klass);
@@ -209,11 +210,6 @@ rb_query_creator_constructor (GType type, guint n_construct_properties,
 	GObjectClass *parent_class;  
 	GladeXML *xml;
 	GtkWidget *mainbox;
-	GtkBox *hbox;
-	GtkWidget *first_option;
-	GtkWidget *first_criteria;
-	GtkWidget *first_entry;
-
 
 	klass = RB_QUERY_CREATOR_CLASS (g_type_class_peek (type));
 	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
@@ -254,35 +250,19 @@ rb_query_creator_constructor (GType type, guint n_construct_properties,
 	dlg->priv->limit_check = GTK_WIDGET (glade_xml_get_widget (xml, "limitCheck"));
 	dlg->priv->limit_entry = GTK_WIDGET (glade_xml_get_widget (xml, "limitEntry"));
 	dlg->priv->limit_option = GTK_WIDGET (glade_xml_get_widget (xml, "limitOption"));
+	dlg->priv->addbutton = GTK_WIDGET (glade_xml_get_widget (xml, "addButton"));
 
 	g_signal_connect_object (G_OBJECT (dlg->priv->limit_check), "toggled", G_CALLBACK (limit_toggled_cb),
 				 dlg, 0);
 	gtk_widget_set_sensitive (dlg->priv->limit_entry, FALSE);
 	gtk_widget_set_sensitive (dlg->priv->limit_option, FALSE);
 
-	dlg->priv->vbox = GTK_BOX (glade_xml_get_widget (xml, "sub_vbox"));
-	dlg->priv->addbutton = gtk_button_new_from_stock (GTK_STOCK_ADD);
 	gtk_size_group_add_widget (dlg->priv->button_size_group, dlg->priv->addbutton);
 	g_signal_connect_object (G_OBJECT (dlg->priv->addbutton), "clicked", G_CALLBACK (add_button_click_cb),
 				 dlg, 0);
-	first_option = create_property_option_menu (dlg, property_options,
-						    G_N_ELEMENTS (property_options),
-						    FALSE);
-	gtk_size_group_add_widget (dlg->priv->property_size_group, first_option);
-	first_criteria = create_criteria_option_menu (string_criteria_options,
-						      G_N_ELEMENTS (string_criteria_options),
-						      FALSE);
-	gtk_size_group_add_widget (dlg->priv->criteria_size_group, first_criteria);
-	first_entry = gtk_entry_new ();
-	gtk_size_group_add_widget (dlg->priv->entry_size_group, first_entry);
-
-	hbox = GTK_BOX (gtk_hbox_new (FALSE, 5));
-	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (first_option));
-	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (first_criteria));
-	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (first_entry));
-	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (dlg->priv->addbutton));
-	gtk_box_pack_start_defaults (dlg->priv->vbox, GTK_WIDGET (hbox));
-	dlg->priv->rows = g_list_prepend (dlg->priv->rows, hbox);
+	
+	dlg->priv->vbox = GTK_BOX (glade_xml_get_widget (xml, "sub_vbox"));
+	append_row(dlg);
 
 	mainbox = glade_xml_get_widget (xml, "main_vbox");
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), mainbox, FALSE, FALSE, 0);
@@ -389,11 +369,12 @@ rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
 
 	subquery = qdata->subquery;
 
-	for (i = 0; i < subquery->len - 1; i++) {
-		RhythmDBQueryData *data = g_ptr_array_index (subquery, i);
-		if (data->type != RHYTHMDB_QUERY_DISJUNCTION)
-			append_row (creator);
-	}
+	if (subquery->len > 0)
+		for (i = 0; i < subquery->len - 1; i++) {
+			RhythmDBQueryData *data = g_ptr_array_index (subquery, i);
+			if (data->type != RHYTHMDB_QUERY_DISJUNCTION)
+				append_row (creator);
+		}
 	
 	rows = creator->priv->rows;
 
@@ -425,7 +406,7 @@ rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
 			gtk_entry_set_text (text, g_value_get_string (data->val));
 		} else {
 			RBRating *rating = RB_RATING (get_box_widget_at_pos (GTK_BOX (rows->data), 2));
-			g_object_set (G_OBJECT (rating), "score",
+			g_object_set (G_OBJECT (rating), "rating",
 				      g_value_get_double (data->val), NULL);
 		}
 		rows = rows->next;
@@ -437,20 +418,14 @@ rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (creator->priv->limit_check),
 				      limit_count || limit_size);
 	if (limit_count) {
-		char *str = g_strdup_printf ("%d", limit_count);
 		gtk_option_menu_set_history (GTK_OPTION_MENU (creator->priv->limit_option), 0);
-		gtk_entry_set_text (GTK_ENTRY (creator->priv->limit_entry), str);
-		g_free (str);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (creator->priv->limit_entry), limit_count);
 	} else if (limit_size > 0 && limit_size < 1000) {
-		char *str = g_strdup_printf ("%d", limit_size);
 		gtk_option_menu_set_history (GTK_OPTION_MENU (creator->priv->limit_option), 1);
-		gtk_entry_set_text (GTK_ENTRY (creator->priv->limit_entry), str);
-		g_free (str);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (creator->priv->limit_entry), limit_size);
 	} else if (limit_size >= 1000) {
-		char *str = g_strdup_printf ("%d", limit_size / 1000);
 		gtk_option_menu_set_history (GTK_OPTION_MENU (creator->priv->limit_option), 2);
-		gtk_entry_set_text (GTK_ENTRY (creator->priv->limit_entry), str);
-		g_free (str);
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (creator->priv->limit_entry), limit_size / 1000);
 	}
 }
 
@@ -512,7 +487,7 @@ get_criteria_for_property (RBQueryCreator *creator, RhythmDBPropType prop, guint
 static void
 set_rating_score (RBRating *rating, gdouble score)
 {
-	g_object_set (G_OBJECT (rating), "score", score, NULL);
+	g_object_set (G_OBJECT (rating), "rating", score, NULL);
 }	
 
 static GtkWidget *
@@ -580,27 +555,30 @@ rb_query_creator_get_query (RBQueryCreator *dlg)
 			const char *data = gtk_entry_get_text (GTK_ENTRY (text));
 			char *folded_data = criteria_option->strict ? g_strdup (data) : g_utf8_casefold (data, -1);
 			
-			if (disjunction && row->next)
-				rhythmdb_query_append (dlg->priv->db,
-						       sub_query,
-						       criteria,
-						       prop,
-						       folded_data,
-						       RHYTHMDB_QUERY_DISJUNCTION,
-						       RHYTHMDB_QUERY_END);
-			else
-				rhythmdb_query_append (dlg->priv->db,
-						       sub_query,
-						       criteria,
-						       prop,
-						       folded_data,
-						       RHYTHMDB_QUERY_END);
+			/* ignore the row if the string is empty */
+			if (folded_data && *folded_data) {
+				if (disjunction && row->next)
+					rhythmdb_query_append (dlg->priv->db,
+							       sub_query,
+							       criteria,
+							       prop,
+							       folded_data,
+							       RHYTHMDB_QUERY_DISJUNCTION,
+							       RHYTHMDB_QUERY_END);
+				else
+					rhythmdb_query_append (dlg->priv->db,
+							       sub_query,
+							       criteria,
+							       prop,
+							       folded_data,
+							       RHYTHMDB_QUERY_END);
+			}
 			g_free (folded_data);
 		} else {
 			double rating;
 
 			g_object_get (G_OBJECT (get_box_widget_at_pos (GTK_BOX (row->data), 2)),
-				      "score", &rating, NULL);
+				      "rating", &rating, NULL);
 			if (disjunction && row->next)
 				rhythmdb_query_append (dlg->priv->db,
 						       sub_query,
@@ -632,7 +610,7 @@ rb_query_creator_get_limit (RBQueryCreator *dlg, RBQueryCreatorLimitType *type,
 {
 	guint limitpos;
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dlg->priv->limit_check)))
-		*limit = atoi (gtk_entry_get_text (GTK_ENTRY (dlg->priv->limit_entry)));
+		*limit = (guint) gtk_spin_button_get_value(GTK_SPIN_BUTTON (dlg->priv->limit_entry));
 	else
 		*limit = 0;
 	limitpos = gtk_option_menu_get_history (GTK_OPTION_MENU (dlg->priv->limit_option));
@@ -707,23 +685,12 @@ append_row (RBQueryCreator *dialog)
 	GtkWidget *criteria;
 	GtkWidget *entry;
 	GtkWidget *remove_button;
-	GtkBox *last_hbox;
 	GtkBox *hbox;
 	GList *rows;
 	guint len;
 
 	rows = dialog->priv->rows;
 	len = g_list_length (rows);
-	last_hbox = GTK_BOX (get_box_widget_at_pos (dialog->priv->vbox, len-1));
-	g_object_ref (G_OBJECT (dialog->priv->addbutton));
-	gtk_container_remove (GTK_CONTAINER (last_hbox),
-			      dialog->priv->addbutton);
-
-	remove_button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
-	g_signal_connect_object (G_OBJECT (remove_button), "clicked", G_CALLBACK (remove_button_click_cb),
-				 dialog, 0);
-	gtk_size_group_add_widget (dialog->priv->button_size_group, remove_button);
-	gtk_box_pack_start_defaults (last_hbox, GTK_WIDGET (remove_button));
 
 	hbox = GTK_BOX (gtk_hbox_new (FALSE, 5));
 	gtk_box_pack_start_defaults (GTK_BOX (dialog->priv->vbox), GTK_WIDGET (hbox));
@@ -747,8 +714,11 @@ append_row (RBQueryCreator *dialog)
 	gtk_size_group_add_widget (dialog->priv->entry_size_group, entry);
 	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (entry));
 
-	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (dialog->priv->addbutton));
-	g_object_unref (G_OBJECT (dialog->priv->addbutton));
+	remove_button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
+	g_signal_connect_object (G_OBJECT (remove_button), "clicked", G_CALLBACK (remove_button_click_cb),
+				 dialog, 0);
+	gtk_size_group_add_widget (dialog->priv->button_size_group, remove_button);
+	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (remove_button));
 
 	gtk_widget_show_all (GTK_WIDGET (dialog->priv->vbox));
 	return GTK_WIDGET (hbox);
@@ -778,7 +748,7 @@ select_criteria_from_value (RBQueryCreator *creator,
 {
 	int i;
 	const RBQueryCreatorCriteriaOption *options;
-	gint length;
+	guint length;
 
 	options = get_criteria_for_property (creator, prop, &length);
 			
@@ -800,7 +770,7 @@ property_option_menu_changed (GtkOptionMenu *propmenu,
 	GtkWidget *entry;
 	const RBQueryCreatorPropertyOption *prop_option;
 	const RBQueryCreatorCriteriaOption *criteria_options;
-	int length;
+	guint length;
 	gboolean constrain;
 
 	row = lookup_row_by_widget (creator, GTK_WIDGET (propmenu));
