@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  arch-tag: Implementation of local song properties dialog
  *
@@ -39,7 +40,6 @@
 #include "rb-glade-helpers.h"
 #include "rb-dialog.h"
 #include "rb-rating.h"
-#include "rb-ellipsizing-label.h"
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
 
@@ -48,6 +48,7 @@ static void rb_song_info_init (RBSongInfo *song_info);
 static GObject *rb_song_info_constructor (GType type, guint n_construct_properties,
 					  GObjectConstructParam *construct_properties);
 
+static void rb_song_info_show (GtkWidget *widget);
 static void rb_song_info_finalize (GObject *object);
 static void rb_song_info_set_property (GObject *object, 
 				       guint prop_id,
@@ -69,6 +70,7 @@ static void rb_song_info_update_bitrate (RBSongInfo *song_info);
 static void rb_song_info_update_buttons (RBSongInfo *song_info);
 static void rb_song_info_update_auto_rate (RBSongInfo *song_info);
 static void rb_song_info_update_rating (RBSongInfo *song_info);
+static void rb_song_info_update_playback_error (RBSongInfo *song_info);
 
 static void rb_song_info_backward_clicked_cb (GtkWidget *button,
 					      RBSongInfo *song_info);
@@ -113,6 +115,8 @@ struct RBSongInfoPrivate
 	GtkWidget   *genre_label;
 	GtkWidget   *track_cur;
 	GtkWidget   *track_cur_label;
+	GtkWidget   *playback_error_box;
+	GtkWidget   *playback_error_label;
 
 	GtkWidget   *bitrate;
 	GtkWidget   *duration;
@@ -175,6 +179,7 @@ static void
 rb_song_info_class_init (RBSongInfoClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -182,6 +187,8 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 	object_class->get_property = rb_song_info_get_property;
 	object_class->constructor = rb_song_info_constructor;
 
+	widget_class->show = rb_song_info_show;
+
 	g_object_class_install_property (object_class,
 					 PROP_ENTRY_VIEW,
 					 g_param_spec_object ("entry_view",
@@ -218,8 +225,6 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 static void
 rb_song_info_init (RBSongInfo *song_info)
 {
-	GtkWidget *close;
-	
 	/* create the dialog and some buttons backward - forward - close */
 	song_info->priv = g_new0 (RBSongInfoPrivate, 1);
 
@@ -230,17 +235,27 @@ rb_song_info_init (RBSongInfo *song_info)
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (song_info), FALSE);
 
-	close = gtk_dialog_add_button (GTK_DIALOG (song_info),
-				       GTK_STOCK_CLOSE,
-				       GTK_RESPONSE_CLOSE);
+	gtk_dialog_add_button (GTK_DIALOG (song_info),
+			       GTK_STOCK_CLOSE,
+			       GTK_RESPONSE_CLOSE);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (song_info),
 					 GTK_RESPONSE_CLOSE);
 
+
+
 	gtk_container_set_border_width (GTK_CONTAINER (song_info), 5);
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (song_info)->vbox), 2);
 }
 
+static void
+rb_song_info_show (GtkWidget *widget)
+{
+	if (GTK_WIDGET_CLASS (parent_class)->show)
+		GTK_WIDGET_CLASS (parent_class)->show (widget);
+
+	rb_song_info_update_playback_error (RB_SONG_INFO (widget));
+}
 
 static void
 rb_song_info_construct_single (RBSongInfo *song_info, GladeXML *xml,
@@ -383,14 +398,12 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 
 	g_return_val_if_fail (selected_entries != NULL, NULL);
 
-	rhythmdb_read_lock (song_info->priv->db);
 	for (tem = selected_entries; tem; tem = tem->next)
 		if (!rhythmdb_entry_is_editable (song_info->priv->db,
 						 selected_entries->data)) {
 			editable = FALSE;
 			break;
 		}
-	rhythmdb_read_unlock (song_info->priv->db);
 	song_info->priv->editable = editable;
 
 	if (selected_entries->next == NULL) {
@@ -423,6 +436,8 @@ rb_song_info_constructor (GType type, guint n_construct_properties,
 	song_info->priv->album = glade_xml_get_widget (xml, "song_info_album");
 	song_info->priv->genre = glade_xml_get_widget (xml, "song_info_genre");
 	song_info->priv->auto_rate = glade_xml_get_widget (xml, "song_info_auto_rate");
+	song_info->priv->playback_error_box = glade_xml_get_widget (xml, "song_info_error_box");
+	song_info->priv->playback_error_label = glade_xml_get_widget (xml, "song_info_error_label");
 
         /* We add now the Pango attributes (look at bug #99867 and #97061) */
 	{
@@ -640,7 +655,7 @@ rb_song_info_set_entry_auto_rate (RBSongInfo *song_info,
 	/* set the new value for auto-rate */
 	g_value_init (&value, G_TYPE_BOOLEAN);
 	g_value_set_boolean (&value, active);
-	rhythmdb_entry_set (song_info->priv->db,
+	rhythmdb_entry_sync (song_info->priv->db,
 			    entry,
 			    RHYTHMDB_PROP_AUTO_RATE,
 			    &value);
@@ -658,11 +673,10 @@ rb_song_info_auto_rate_toggled_cb (GtkToggleButton *togglebutton,
 	g_return_if_fail (GTK_IS_TOGGLE_BUTTON (togglebutton));
 
 	active = gtk_toggle_button_get_active (togglebutton);
-	rhythmdb_write_lock (song_info->priv->db);
 	rb_song_info_selection_for_each (song_info,
 					 rb_song_info_set_entry_auto_rate,
 					 &active);
-	rhythmdb_write_unlock (song_info->priv->db);
+	rhythmdb_commit (song_info->priv->db);
 }
 
 static void
@@ -676,7 +690,7 @@ rb_song_info_set_entry_rating (RBSongInfo *info,
 	/* set the new value for the song */
 	g_value_init (&value, G_TYPE_DOUBLE);
 	g_value_set_double (&value, trouble);
-	rhythmdb_entry_set (info->priv->db,
+	rhythmdb_entry_sync (info->priv->db,
 			    entry,
 			    RHYTHMDB_PROP_RATING,
 			    &value);
@@ -684,7 +698,7 @@ rb_song_info_set_entry_rating (RBSongInfo *info,
 	/* since the user changed the rating, stop auto-rating */
 	g_value_init (&value, G_TYPE_BOOLEAN);
 	g_value_set_boolean (&value, FALSE);
-	rhythmdb_entry_set (info->priv->db,
+	rhythmdb_entry_sync (info->priv->db,
 			    entry,
 			    RHYTHMDB_PROP_AUTO_RATE,
 			    &value);
@@ -702,14 +716,13 @@ rb_song_info_rated_cb (RBRating *rating,
 	g_return_if_fail (RB_IS_SONG_INFO (song_info));
 	g_return_if_fail (score >= 0 && score <= 5 );
 
-	rhythmdb_write_lock (song_info->priv->db);
 	rb_song_info_selection_for_each (song_info,
 					 rb_song_info_set_entry_rating,
 					 &score);
-	rhythmdb_write_unlock (song_info->priv->db);
+	rhythmdb_commit (song_info->priv->db);
 
 	g_object_set (G_OBJECT (song_info->priv->rating),
-		      "score", score,
+		      "rating", score,
 		      NULL);
 
 	rb_song_info_update_auto_rate (song_info);
@@ -732,7 +745,7 @@ rb_song_info_mnemonic_cb (GtkWidget *target)
 static void 
 rb_song_info_populate_dialog (RBSongInfo *song_info)
 {
-	gint num;
+	gulong num;
 	const char *text = NULL;
 	char *tmp;
 	/* update the buttons sensitivity */
@@ -741,37 +754,26 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 	if (!song_info->priv->current_entry)
 		return;
 	
-	rhythmdb_read_lock (song_info->priv->db);
 
-	text = rhythmdb_entry_get_string (song_info->priv->db,
-					  song_info->priv->current_entry,
-					  RHYTHMDB_PROP_TITLE);
+	text = rb_refstring_get (song_info->priv->current_entry->title);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->title), text);
 	
 	tmp = g_strdup_printf (_("%s Properties"), text);
 	gtk_window_set_title (GTK_WINDOW (song_info), tmp);
 	g_free (tmp);
 
-	text = rhythmdb_entry_get_string (song_info->priv->db,
-					  song_info->priv->current_entry,
-					  RHYTHMDB_PROP_ARTIST);
+	text = rb_refstring_get (song_info->priv->current_entry->artist);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->artist), text);
-	text = rhythmdb_entry_get_string (song_info->priv->db,
-					  song_info->priv->current_entry,
-					  RHYTHMDB_PROP_ALBUM);
+	text = rb_refstring_get (song_info->priv->current_entry->album);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->album), text);
-	text = rhythmdb_entry_get_string (song_info->priv->db,
-					  song_info->priv->current_entry,
-					  RHYTHMDB_PROP_GENRE);
+	text = rb_refstring_get (song_info->priv->current_entry->genre);
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->genre), text);
 
-	num = rhythmdb_entry_get_int (song_info->priv->db,
-				      song_info->priv->current_entry,
-				      RHYTHMDB_PROP_TRACK_NUMBER);
+	num = song_info->priv->current_entry->tracknum;
 	if (num > 0)
-		tmp = g_strdup_printf ("%.2d", num);
+		tmp = g_strdup_printf ("%.2ld", num);
 	else
-		tmp = g_strdup (_("Never"));
+		tmp = g_strdup (_("Unknown"));
 	gtk_entry_set_text (GTK_ENTRY (song_info->priv->track_cur),
 			    tmp);
 	g_free (tmp);
@@ -783,20 +785,42 @@ rb_song_info_populate_dialog (RBSongInfo *song_info)
 	rb_song_info_update_bitrate (song_info);
 	rb_song_info_update_auto_rate (song_info);
 	rb_song_info_update_rating (song_info);
-	rhythmdb_read_unlock (song_info->priv->db);
+	rb_song_info_update_playback_error (song_info);
+}
+
+static void
+rb_song_info_update_playback_error (RBSongInfo *song_info)
+{
+	char *message = NULL;
+
+	if (!song_info->priv->current_entry)
+		return;
+
+	if (song_info->priv->current_entry->playback_error)
+		message = g_strdup (song_info->priv->current_entry->playback_error);
+
+	if (message) {
+		gtk_label_set_text (GTK_LABEL (song_info->priv->playback_error_label),
+				    message);
+		gtk_widget_show (song_info->priv->playback_error_box);
+	} else {
+		gtk_label_set_text (GTK_LABEL (song_info->priv->playback_error_label),
+				    "No errors");
+		gtk_widget_hide (song_info->priv->playback_error_box);
+	}
+
+	g_free (message);
 }
 
 static void
 rb_song_info_update_bitrate (RBSongInfo *song_info)
 {
 	char *tmp = NULL;
-	int bitrate = 0;
-	bitrate = rhythmdb_entry_get_int (song_info->priv->db,
-					  song_info->priv->current_entry,
-					  RHYTHMDB_PROP_BITRATE);
+	gulong bitrate = 0;
+	bitrate = song_info->priv->current_entry->bitrate;
 
 	if (bitrate > 0)
-		tmp = g_strdup_printf (_("%d kbps"), bitrate);
+		tmp = g_strdup_printf (_("%lu kbps"), bitrate);
 	else
 		tmp = g_strdup (_("Unknown"));
 	gtk_label_set_text (GTK_LABEL (song_info->priv->bitrate),
@@ -810,9 +834,7 @@ rb_song_info_update_duration (RBSongInfo *song_info)
 	char *text = NULL;
 	long duration = 0;
 	int minutes, seconds;
-	duration = rhythmdb_entry_get_long (song_info->priv->db,
-					    song_info->priv->current_entry,
-					    RHYTHMDB_PROP_DURATION);
+	duration = song_info->priv->current_entry->duration;
 	minutes = duration / 60;
 	seconds = duration % 60;
 	text = g_strdup_printf ("%d:%02d", minutes, seconds);
@@ -824,28 +846,29 @@ static void
 rb_song_info_update_location (RBSongInfo *song_info)
 {
 	const char *text;
-	char *basename, *dir, *desktopdir;
 
 	g_return_if_fail (song_info != NULL);
 
-	rhythmdb_read_lock (song_info->priv->db);
-	text = rhythmdb_entry_get_string (song_info->priv->db,
-					  song_info->priv->current_entry,
-					  RHYTHMDB_PROP_LOCATION);
-	rhythmdb_read_unlock (song_info->priv->db);
+	text = song_info->priv->current_entry->location;
 
 	if (text != NULL) {
-		char *tmp;
+		char *tmp, *tmp_utf8;
+		char *basename, *dir, *desktopdir;
 		
 		basename = g_path_get_basename (text);
 		tmp = gnome_vfs_unescape_string_for_display (basename);
 		g_free (basename);
-
-		if (tmp != NULL) {
-			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name), tmp);
+		tmp_utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
+		g_free (tmp);
+		if (tmp_utf8 != NULL) {
+			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name), 
+					    tmp_utf8);
+		} else {
+			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name), 
+					    _("Unknown file name"));
 		}
 
-		g_free (tmp);
+		g_free (tmp_utf8);
 	
 		tmp = gnome_vfs_get_local_path_from_uri (text);
 		if (tmp == NULL)
@@ -854,17 +877,25 @@ rb_song_info_update_location (RBSongInfo *song_info)
 		g_free (tmp);
 		tmp = gnome_vfs_unescape_string_for_display (dir);
 		g_free (dir);
+		tmp_utf8 = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
+		g_free (tmp);
 
-		desktopdir = g_build_filename (g_get_home_dir (), ".gnome-desktop", NULL);
-		if (strcmp (tmp, desktopdir) == 0)
+		desktopdir = g_build_filename (g_get_home_dir (), "Desktop", NULL);
+		if ((tmp_utf8 != NULL) && (strcmp (tmp_utf8, desktopdir) == 0))
 		{
-			g_free (tmp);
-			tmp = g_strdup (_("on the desktop"));
+			g_free (tmp_utf8);
+			tmp_utf8 = g_strdup (_("On the desktop"));
 		}
 		g_free (desktopdir);
-		
-		gtk_entry_set_text (GTK_ENTRY (song_info->priv->location), tmp);
-		g_free (tmp);
+
+		if (tmp_utf8 != NULL) {
+			gtk_entry_set_text (GTK_ENTRY (song_info->priv->location), 
+					    tmp_utf8);
+		} else {
+			gtk_entry_set_text (GTK_ENTRY (song_info->priv->location), 
+					    _("Unknown location"));
+		}
+		g_free (tmp_utf8);
 	}
 }
 
@@ -936,9 +967,7 @@ static void
 rb_song_info_update_play_count (RBSongInfo *song_info)
 {
 	char *text;
-	text = g_strdup_printf ("%d", rhythmdb_entry_get_int (song_info->priv->db,
-							      song_info->priv->current_entry,
-							      RHYTHMDB_PROP_PLAY_COUNT));
+	text = g_strdup_printf ("%ld", song_info->priv->current_entry->play_count);
 	gtk_label_set_text (GTK_LABEL (song_info->priv->play_count), text);
 	g_free (text);
 }
@@ -947,9 +976,7 @@ static void
 rb_song_info_update_last_played (RBSongInfo *song_info)
 {
 	const char *str;
-	str = rhythmdb_entry_get_string (song_info->priv->db,
-					 song_info->priv->current_entry,
-					 RHYTHMDB_PROP_LAST_PLAYED_STR);
+	str = rb_refstring_get (song_info->priv->current_entry->last_played_str);
 	if (!strcmp ("", str))
 		str = _("Never");
 	gtk_label_set_text (GTK_LABEL (song_info->priv->last_played), str);
@@ -963,9 +990,7 @@ rb_song_info_update_auto_rate_single (RBSongInfo *song_info)
 	
 	g_return_if_fail (RB_IS_SONG_INFO (song_info));
 
-	auto_rate = rhythmdb_entry_get_boolean (song_info->priv->db,
-					    song_info->priv->current_entry,
-					    RHYTHMDB_PROP_AUTO_RATE);
+	auto_rate = song_info->priv->current_entry->auto_rate;
 
 	/* We have to block our signal handlers from thinking this
 	   is a user-originated setting */
@@ -989,18 +1014,15 @@ rb_song_info_update_auto_rate_multiple (RBSongInfo *song_info)
 	gboolean global_auto_rate;
 	gboolean inconsistent = FALSE;
 	GList *tem;
+	RhythmDBEntry *entry;
 	
 	g_return_if_fail (RB_IS_SONG_INFO (song_info));
 
-	first_auto_rate = rhythmdb_entry_get_boolean (song_info->priv->db,
-						      song_info->priv->selected_entries->data,
-						      RHYTHMDB_PROP_AUTO_RATE);
+	entry = song_info->priv->selected_entries->data;
+	first_auto_rate = entry->auto_rate;
 	for (tem = song_info->priv->selected_entries; tem; tem = tem->next) {
-		gboolean auto_rate
-			= rhythmdb_entry_get_boolean (song_info->priv->db,
-						      tem->data,
-						      RHYTHMDB_PROP_AUTO_RATE);
-		if (auto_rate != first_auto_rate) {
+		entry = tem->data;
+		if (entry->auto_rate != first_auto_rate) {
 			inconsistent = TRUE;
 			break;
 		}
@@ -1038,16 +1060,10 @@ rb_song_info_update_auto_rate (RBSongInfo *song_info)
 static void
 rb_song_info_update_rating (RBSongInfo *song_info)
 {
-	gdouble rating;
-	
 	g_return_if_fail (RB_IS_SONG_INFO (song_info));
 
-	rating = rhythmdb_entry_get_double (song_info->priv->db,
-					    song_info->priv->current_entry,
-					    RHYTHMDB_PROP_RATING);
-
 	g_object_set (G_OBJECT (song_info->priv->rating),
-		      "score", rating,
+		      "rating", song_info->priv->current_entry->rating,
 		      NULL);
 
 }
@@ -1060,37 +1076,53 @@ rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 	const char *album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));	
 	GValue val = {0,};
 	GList *tem;
-
-	rhythmdb_write_lock (dialog->priv->db);
+	gboolean changed = FALSE;
+	RhythmDBEntry *entry;
 
 	if (strlen (album) > 0) {
 		g_value_init (&val, G_TYPE_STRING);
 		g_value_set_string (&val, album);
-		for (tem = dialog->priv->selected_entries; tem; tem = tem->next)
-			rhythmdb_entry_set (dialog->priv->db,
-					    tem->data, RHYTHMDB_PROP_ALBUM, &val);
+		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
+			entry = (RhythmDBEntry *)tem->data;
+			if (strcmp (album, rb_refstring_get (entry->album)) == 0)
+				continue;
+			rhythmdb_entry_sync (dialog->priv->db, entry,
+					     RHYTHMDB_PROP_ALBUM, &val);
+			changed = TRUE;
+		}
 		g_value_unset (&val);
 	}
 	
 	if (strlen (artist) > 0) {
 		g_value_init (&val, G_TYPE_STRING);
 		g_value_set_string (&val, artist);
-		for (tem = dialog->priv->selected_entries; tem; tem = tem->next)
-			rhythmdb_entry_set (dialog->priv->db,
-					    tem->data, RHYTHMDB_PROP_ARTIST, &val);
+		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
+			entry = (RhythmDBEntry *)tem->data;
+			if (strcmp (artist, rb_refstring_get (entry->artist)) == 0)
+				continue;
+			rhythmdb_entry_sync (dialog->priv->db, entry,
+					     RHYTHMDB_PROP_ARTIST, &val);
+			changed = TRUE;
+		}
 		g_value_unset (&val);
 	}
 
 	if (strlen (genre) > 0) {
 		g_value_init (&val, G_TYPE_STRING);
 		g_value_set_string (&val, genre);
-		for (tem = dialog->priv->selected_entries; tem; tem = tem->next)
-			rhythmdb_entry_set (dialog->priv->db,
-					    tem->data, RHYTHMDB_PROP_GENRE, &val);
+		for (tem = dialog->priv->selected_entries; tem; tem = tem->next) {
+			entry = (RhythmDBEntry *)tem->data;
+			if (strcmp (genre, rb_refstring_get (entry->genre)) == 0)
+				continue;
+			rhythmdb_entry_sync (dialog->priv->db, entry,
+					     RHYTHMDB_PROP_GENRE, &val);
+			changed = TRUE;
+		}
 		g_value_unset (&val);
 	}
-	
-	rhythmdb_write_unlock (dialog->priv->db);
+
+	if (changed)
+		rhythmdb_commit (dialog->priv->db);
 }
 
 static void
@@ -1102,51 +1134,81 @@ rb_song_info_sync_entry_single (RBSongInfo *dialog)
 	const char *album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));	
 	const char *tracknum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->track_cur));
 	char *endptr;
+	GType type;
 	gint tracknum;
 	GValue val = {0,};
-
-	tracknum = g_ascii_strtoull (tracknum_str, &endptr, 10);
-	if (endptr == tracknum_str)
-		tracknum = -1;
-
-	rhythmdb_write_lock (dialog->priv->db);
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_string (&val, title);
-	rhythmdb_entry_set (dialog->priv->db,
-			    dialog->priv->current_entry, RHYTHMDB_PROP_TITLE, &val);
-	g_value_unset (&val);
-
-	g_value_init (&val, G_TYPE_INT);
-	g_value_set_int (&val, tracknum);
-	rhythmdb_entry_set (dialog->priv->db,
-			    dialog->priv->current_entry, RHYTHMDB_PROP_TRACK_NUMBER, &val);
-	g_value_unset (&val);
+	gboolean changed = FALSE;
+	RhythmDBEntry *entry = dialog->priv->current_entry;
 
 	g_signal_emit (G_OBJECT (dialog), rb_song_info_signals[PRE_METADATA_CHANGE], 0,
-		       dialog->priv->current_entry);
+		       entry);
 
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_string (&val, album);
-	rhythmdb_entry_set (dialog->priv->db,
-			    dialog->priv->current_entry, RHYTHMDB_PROP_ALBUM, &val);
-	g_value_unset (&val);
+	tracknum = g_ascii_strtoull (tracknum_str, &endptr, 10);
+	if ((endptr != tracknum_str) && (tracknum != entry->tracknum)) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_TRACK_NUMBER);
+		g_value_init (&val, type);
+		g_value_set_ulong (&val, tracknum);
+		rhythmdb_entry_sync (dialog->priv->db, entry,
+				     RHYTHMDB_PROP_TRACK_NUMBER, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
 
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_string (&val, artist);
-	rhythmdb_entry_set (dialog->priv->db,
-			    dialog->priv->current_entry, RHYTHMDB_PROP_ARTIST, &val);
-	g_value_unset (&val);
+	if (strcmp (title, rb_refstring_get (entry->title))) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_TITLE);
+		g_value_init (&val, type);
+		g_value_set_string (&val, title);
+		rhythmdb_entry_sync (dialog->priv->db, entry,
+				     RHYTHMDB_PROP_TITLE, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
 
-	g_value_init (&val, G_TYPE_STRING);
-	g_value_set_string (&val, genre);
-	rhythmdb_entry_set (dialog->priv->db,
-			    dialog->priv->current_entry, RHYTHMDB_PROP_GENRE, &val);
-	g_value_unset (&val);
 
+	if (strcmp (album, rb_refstring_get (entry->album))) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_ALBUM);
+		g_value_init (&val, type);
+		g_value_set_string (&val, album);
+		rhythmdb_entry_sync (dialog->priv->db, entry,
+				     RHYTHMDB_PROP_ALBUM, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
+
+	if (strcmp (artist, rb_refstring_get (entry->artist))) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_ARTIST);
+		g_value_init (&val, type);
+		g_value_set_string (&val, artist);
+		rhythmdb_entry_sync (dialog->priv->db, entry,
+				     RHYTHMDB_PROP_ARTIST, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
+
+
+	if (strcmp (artist, rb_refstring_get (entry->artist))) {
+		type = rhythmdb_get_property_type (dialog->priv->db,
+						   RHYTHMDB_PROP_GENRE);
+		g_value_init (&val, type);
+		g_value_set_string (&val, genre);
+		rhythmdb_entry_sync (dialog->priv->db, entry,
+				     RHYTHMDB_PROP_GENRE, &val);
+		g_value_unset (&val);
+		changed = TRUE;
+	}
+
+	/* FIXME: when an entry is SYNCed, a changed signal is emitted, and
+	 * this signal is also emitted, aren't they redundant?
+	 */
 	g_signal_emit (G_OBJECT (dialog), rb_song_info_signals[POST_METADATA_CHANGE], 0,
-		       dialog->priv->current_entry);
+		       entry);
 
-	rhythmdb_write_unlock (dialog->priv->db);
+	if (changed)
+		rhythmdb_commit (dialog->priv->db);
 }	
 
 static void
